#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

import re
import copy
import math
import logging

from parse_datacmds import parse_data_commands
from pyutilib.misc import quote_split, Options
from coopr.pyomo.base.plugin import *
from coopr.pyomo.base.plugin import *
import pyutilib.common

logger = logging.getLogger('coopr.pyomo')

global Lineno
global Filename


def _preprocess_data(cmd):
        """
        Called by _process_data() to (1) combine tokens that comprise a tuple
        and (2) combine the ':' token with the previous token
        """
        if __debug__:
            logger.debug("_preprocess_data(start) %s",cmd)
        status=")"
        newcmd=[]
        for token in cmd:
          if type(token) in (str,unicode):
             token=str(token)
             if "(" in token and ")" in token:
                newcmd.append(token)
                status=")"
             elif "(" in token:
                if status == "(":
                   raise ValueError, "Two '('s follow each other in data "+token
                status="("
                newcmd.append(token)
             elif ")" in token:
                if status == ")":
                   raise ValueError, "Two ')'s follow each other in data"
                status=")"
                newcmd[-1] = newcmd[-1]+token
             elif status == "(":
                newcmd[-1] = newcmd[-1]+token
             else:
                newcmd.append(token)
          else:
             if type(token) is float and math.floor(token) == token:
                token=int(token)
             newcmd.append(token)
        if __debug__:
            logger.debug("_preprocess_data(end) %s", newcmd)
        return newcmd


def _process_set(cmd, _model, _data):
        """
        Called by _process_data() to process a set declaration.
        """
        if __debug__:
            logger.debug("DEBUG: _process_set(start) %s",cmd)
        #
        # Process a set
        #
        if "[" in cmd[1]:
           tokens = re.split("[\[\]]",cmd[1])
           ndx=tokens[1]
           ndx=tuple(_data_eval(ndx.split(",")))
           if tokens[0] not in _data:
              _data[tokens[0]] = {}
           _data[tokens[0]][ndx] = _process_set_data(cmd[3:], tokens[0], _model)
        elif cmd[2] == ":":
           _data[cmd[1]] = {}
           _data[cmd[1]][None] = []
           i=3
           while cmd[i] != ":=":
              i += 1
           ndx1 = cmd[3:i]
           i += 1
           while i<len(cmd):
              ndx=cmd[i]
              for j in range(0,len(ndx1)):
                if cmd[i+j+1] == "+":
                   _data[cmd[1]][None] += _process_set_data(["("+str(ndx1[j])+","+str(cmd[i])+")"], cmd[1], _model)
              i += len(ndx1)+1
        else:
           _data[cmd[1]] = {}
           _data[cmd[1]][None] = _process_set_data(cmd[3:], cmd[1], _model)


def _process_set_data(cmd, sname, _model):
        """
        Called by _process_set() to process set data.
        """
        if __debug__:
            logger.debug("DEBUG: _process_set_data(start) %s",cmd)
        if len(cmd) == 0:
           return []
        try:
            sd = getattr(_model,sname)
        except AttributeError:
            raise IOError, "Cannot process data for set '%s' because it is not defined in model '%s'" % (sname, _model.name)
        #d = sd.dimen
        cmd = _data_eval(cmd)
        ans=[]
        i=0
        tmp=None
        ndx=[]
        while i<len(cmd):
          if type(cmd[i]) is not tuple:
            if len(ndx) > 0:
               #if type(cmd[i]) is not tuple:
               #   raise ValueError, "Problem initializing set="+sname+" with input data="+str(cmd)+" - first element was interpreted as a tuple, but element="+str(i)+" is of type="+str(type(cmd[i]))+"; types must be consistent"
               tmpval=tmp
               for kk in range(len(ndx)):
                    if i == len(cmd):
                        raise IOError, "Expected another set value to flush out a tuple pattern!"
                    tmpval[ndx[kk]] = _data_eval([cmd[i]])[0]
                    i += 1
               #
               # WEH - I'm not sure what the next two lines are for
               #        These are called when initializing a set with more than
               #        one dimension
               #if d > 1:
               #   tmpval = util.tuplize(tmpval,d,sname)
               ans.append(tuple(tmpval))
               continue
            else:
               ans.append(cmd[i])
          elif "*" not in cmd[i]:
            ans.append(cmd[i])
          else:
            j = i
            tmp=list(cmd[j])
            ndx=[]
            for kk in range(len(tmp)):
                if tmp[kk] == '*':
                    ndx.append(kk)
            #print 'NDX',ndx
          i += 1
        if __debug__:
            logger.debug("DEBUG: _process_set_data(end) %s",ans)
        return ans


def _process_param(cmd, _model, _data, _default):
        """
        Called by _process_data to process data for a Parameter declaration
        """
        #print 'x',cmd,_data
        if __debug__:
            logger.debug("DEBUG: _process_param(start) %s",cmd)
        #
        # Process parameters
        #
        dflt = None
        singledef = True
        cmd = cmd[1:]
        if cmd[0] == ":":
           singledef = False
           cmd = cmd[1:]
        if singledef:
           pname = cmd[0]
           cmd = cmd[1:]
           if len(cmd) >= 2 and cmd[0] == "default":
              dflt = _data_eval(cmd[1])[0]
              cmd = cmd[2:]
           if dflt != None:
              _default[pname] = dflt
           if cmd[0] == ":=":
              cmd = cmd[1:]
           transpose = False
           if cmd[0] == "(tr)":
              transpose = True
              if cmd[1] == ":":
                 cmd = cmd[1:]
              else:
                 cmd[0] = ":"
           if cmd[0] != ":":
              if __debug__:
                    logger.debug("DEBUG: _process_param (singledef without :...:=) %s",cmd)
              cmd = _apply_templates(cmd)
              #print 'cmd',cmd
              if not transpose:
                 if pname not in _data:
                    _data[pname] = {}
                 finaldata = _process_data_list(getattr(_model,pname).dim(), _data_eval(cmd))
                 for key in finaldata:
                    _data[pname][key]=finaldata[key]
              else:
                 tmp = ["param", pname, ":="]
                 i=1
                 while i < len(cmd):
                    i0 = i
                    while cmd[i] != ":=":
                      i=i+1
                    ncol = i - i0 + 1
                    lcmd = i
                    while lcmd < len(cmd) and cmd[lcmd] != ":":
                      lcmd += 1
                    j0 = i0 - 1
                    for j in range(1,ncol):
                      ii = 1 + i
                      kk = ii + j
                      while kk < lcmd:
                        if cmd[kk] != ".":
                        #if 1>0:
                           tmp.append(copy.copy(cmd[j+j0]))
                           tmp.append(copy.copy(cmd[ii]))
                           tmp.append(copy.copy(cmd[kk]))
                        ii = ii + ncol
                        kk = kk + ncol
                    i = lcmd + 1
                 _process_param(tmp, _model, _data, _default)
           else:
              tmp = ["param", pname, ":="]
              i=1
              if __debug__:
                 logger.debug("DEBUG: _process_param (singledef with :...:=) %s",cmd)
              while i < len(cmd):
                i0 = i
                while i<len(cmd) and cmd[i] != ":=":
                  i=i+1
                if i==len(cmd):
                   raise ValueError, "ERROR: Trouble on line "+str(Lineno)+" of file "+Filename
                ncol = i - i0 + 1
                lcmd = i
                while lcmd < len(cmd) and cmd[lcmd] != ":":
                  lcmd += 1
                j0 = i0 - 1
                for j in range(1,ncol):
                  ii = 1 + i
                  kk = ii + j
                  while kk < lcmd:
                    if cmd[kk] != ".":
                        if transpose:
                           tmp.append(copy.copy(cmd[j+j0]))
                           tmp.append(copy.copy(cmd[ii]))
                        else:
                           tmp.append(copy.copy(cmd[ii]))
                           tmp.append(copy.copy(cmd[j+j0]))
                        tmp.append(copy.copy(cmd[kk]))
                    ii = ii + ncol
                    kk = kk + ncol
                i = lcmd + 1
                _process_param(tmp, _model, _data, _default)

        else:
           if __debug__:
              logger.debug("DEBUG: _process_param (cmd[0]=='param:') %s",cmd)
           i=0
           nsets=0
           while i<len(cmd) and cmd[i] != ":=":
             if cmd[i] == ":":
                nsets = i
             i += 1
           if i==len(cmd):
              raise ValueError, "Trouble on data file line "+str(Lineno)+" of file "+Filename
           if __debug__:
              logger.debug("NSets %d",nsets)
           Lcmd = len(cmd)
           j=0
           d = 1
           #
           # Process sets first
           #
           while j<nsets:
             sname = cmd[j]
             d = getattr(_model,sname).dimen
             np = i-1
             if __debug__:
                logger.debug("I %d, J %d, SName %s, d %d",i,j,sname,d)
             dnp = d + np - 1
             #k0 = i + d - 2
             ii = i + j + 1
             tmp = [ "set", cmd[j], ":=" ]
             while ii < Lcmd:
               for dd in range(0,d):
                 tmp.append(copy.copy(cmd[ii+dd]))
               ii += dnp
             _process_set(tmp, _model, _data)
             j += 1
           if nsets > 0:
              j += 1
           #
           # Process parameters second
           #
           while j < i:
             pname = cmd[j]
             if __debug__:
                logger.debug("I %d, J %d, Pname %s",i,j,pname)
             #d = 1
             d = getattr(_model,pname).dim()
             if nsets > 0:
                np = i-1
                dnp = d+np-1
                ii = i + 1
                kk = i + d + j-1
             else:
                np = i
                dnp = d + np
                ii = i + 1
                kk = np + 1 + d + nsets + j
             tmp = [ "param", pname, ":=" ]
             if __debug__:
                logger.debug('dnp %d\nnp %d', dnp, np)
             while kk < Lcmd:
               if __debug__:
                  logger.debug("kk %d, ii %d",kk,ii)
               iid = ii + d
               while ii < iid:
                 tmp.append(copy.copy(cmd[ii]))
                 ii += 1
               ii += dnp-d
               tmp.append(copy.copy(cmd[kk]))
               kk += dnp
             _process_param(tmp, _model, _data, _default)
             j += 1


def _apply_templates(cmd):
    cmd = _data_eval(cmd)
    template = []
    ilist = set()
    ans = []
    i = 0
    while i < len(cmd):
        #print i, len(cmd), cmd[i], ilist, template, ans
        if type(cmd[i]) is tuple and '*' in cmd[i]:
            j = i
            tmp=list(cmd[j])
            nindex = len(tmp)
            template=tmp
            ilist = set()
            for kk in range(len(tmp)):
                if tmp[kk] == '*':
                    ilist.add(kk)
        elif len(ilist) == 0:
            ans.append(cmd[i])
        else:
            for kk in range(len(template)):
                if kk in ilist:
                    ans.append(cmd[i])
                    i += 1
                else:
                    ans.append(template[kk])
            ans.append(cmd[i])
        i += 1
    return ans


def _process_data_list(dim, cmd):
   """\
Called by _process_param() to process a list of data for a Parameter.
"""
   if __debug__:
      logger.debug("process_data_list %d %s",dim,cmd)

   if len(cmd) % (dim+1) != 0:
      msg = "Parameter defined with '%d' dimensions, but data has '%d' "   \
            "values: %s"
      raise ValueError, msg % (dim, len(cmd), cmd)

   ans={}
   if dim==0:
      ans[None]=cmd[0]
      return ans
   i=0
   while i < len(cmd):
     ndx = tuple(cmd[i:i+dim])
     if cmd[i+dim] != ".":
            ans[ndx] = cmd[i+dim]
     i += dim+1
   return ans



def _data_eval(values):
        """
        Evaluate the list of values to make them bool, integer or float,
        or a tuple value.
        """
        if __debug__:
           logger.debug("DEBUG: _data_eval(start) %s",values)
        ans = []
        for val in values:
          if type(val) in (bool,int,float,long):
             ans.append(val)
             continue
          if val in ('True','true','TRUE'):
             ans.append(True)
             continue
          if val in ('False','false','FALSE'):
             ans.append(False)
             continue
          tmp = None
          tval = val.strip()
          if (tval[0] == "(" and tval[-1] == ")") or (tval[0] == '[' and tval[-1] == ']'):
             vals = []
             tval = tval[1:-1]
             for item in tval.split(","):
               tmp=_data_eval([item])
               vals.append(tmp[0])
             ans.append(tuple(vals))
             continue
          try:
             tmp = int(val)
             ans.append(tmp)
          except ValueError:
             pass
          if tmp is None:
             try:
               tmp = float(val)
               ans.append(tmp)
             except ValueError:
               if val[0] == "'" or val[0] == '"':
                    ans.append(val[1:-1])
               else:
                    ans.append(val)
        if __debug__:
           logger.debug("DEBUG: _data_eval(end) %s",ans)
        return ans


def _process_include(cmd, _model, _data, _default):
    if len(cmd) == 1:
        raise IOError, "Cannot execute 'include' command without a filename"
    if len(cmd) > 2:
        raise IOError, "The 'include' command only accepts a single filename"

    global Filename
    Filename = cmd[1]
    global Lineno
    Lineno = 0

    try:
        scenarios = parse_data_commands(filename=cmd[1])
    except IOError, err:
        raise IOError, "Error parsing file '%s': %s" % (Filename, str(err))
    if scenarios is None:
        return False
    for scenario in scenarios:
        for cmd in scenarios[scenario]:
            #print "SCENARIO",scenario,"CMD",cmd
            if scenario not in _data:
                _data[scenario] = {}
            #print "X _data",_data
            if cmd[0] in ('include', 'import'):
                _tmpdata = {}
                _process_data(cmd, _model, _tmpdata, _default, Filename, Lineno)
                #print "X _tmpdata",_tmpdata
                if scenario is None:
                    for key in _tmpdata:
                        if key in _data:
                            _data[key].update(_tmpdata[key])
                        else:
                            _data[key] = _tmpdata[key]
                else:
                    for key in _tmpdata:
                        if key is None:
                            _data[scenario].update(_tmpdata[key])
                        else:
                            raise IOError, "Cannot define a scenario within another scenario"
            else:
                _process_data(cmd, _model, _data[scenario], _default, Filename, Lineno)
            #print "X _data update",_data
    return True


def X_process_include(cmd, _model, _data, _default):
    if len(cmd) == 1:
        raise IOError, "Cannot execute 'include' command without a filename"
    if len(cmd) > 2:
        raise IOError, "The 'include' command only accepts a single filename"

    global Filename
    Filename = cmd[1]
    global Lineno
    Lineno = 0
    cmd=""
    status=True
    INPUT=open(Filename,'r')
    for line in INPUT:
        Lineno = Lineno + 1
        line = re.sub(":"," :",line)
        line = line.strip()
        if line == "" or line[0] == '#':
            continue
        cmd = cmd + " " + line
        if ';' in cmd:
            #
            # We assume that a ';' indicates an end-of-command declaration.
            # However, the user might have put multiple commands on a single
            # line, so we need to split the line based on these values.
            # BUT, at the end of the line we should see an 'empty' command,
            # which we ignore.
            #
            for item in cmd.split(';'):
                item = item.strip()
                if item != "":
                    _process_data(quote_split("[\t ]+",item), _model, _data, _default, Filename, Lineno)
                cmd = ""
    if cmd != "":
        INPUT.close()
        raise IOError, "ERROR: There was unprocessed text at the end of the data file!: \"" + cmd + "\""
    INPUT.close()
    return status


def _process_import(cmd, _model, _data, _default):
    #print "IMPORT",cmd
    if len(cmd) < 2:
        raise IOError, "The 'import' command must specify a filename"

    options = Options(**cmd[1])
    for key in options:
        if not key in ['range','filename','format','using','driver','query','table','user','password']:
            raise ValueError, "Unknown import option '%s'" % key

    global Filename
    Filename = cmd[1]
    global Lineno
    Lineno = 0

    #
    # TODO: process mapping info
    #
    if options.using is None:
        tmp = options.filename.split(".")[-1]
        data = DataManagerFactory(tmp)
        if data is None:
            raise pyutilib.common.ApplicationError, "Cannot create data manager '%s'" % tmp
    else:
        data = DataManagerFactory(options.using)
        if data is None:
            raise pyutilib.common.ApplicationError, "Cannot create data manager '%s'" % options.using
    set_name=None
    param_name=None
    #
    # Create symbol map
    #
    symb_map = cmd[3]
    if len(symb_map) == 0:
        raise IOError, "Must specify at least one set or parameter name that will be imported"
    #
    # Process index data
    #
    index=cmd[2][1]
    index_name=cmd[2][0]
    #
    # Set the 'set name' based on the format
    #
    if options.format == 'set' or options.format == 'set_array':
        if len(cmd[3]) != 1:
            raise IOError, "A single set name must be specified when using format '%s'" % options.format
        set_name=cmd[3].keys()[0]
    else:
        set_name=None
    #
    # Set the 'param name' based on the format
    #
    if options.format == 'transposed_array' or options.format == 'array' or options.format == 'param':
        if len(cmd[3]) != 1:
            raise IOError, "A single parameter name must be specified when using format '%s'" % options.format
        param_name = cmd[3].keys()[0]
    else:
        param_name = None
    #
    data.initialize(options.filename, index=index, index_name=index_name, param_name=symb_map, set=set_name, param=param_name, format=options.format, range=options.range, query=options.query, using=options.using, table=options.table)
    #
    data.open()
    try:
        data.read()
    except Exception, e:
	data.close()
	raise e
    data.close()
    data.process(_model, _data, _default)


def _process_data(cmd, _model, _data, _default, Filename_, Lineno_=0):
        """
        Called by import_file() to (1) preprocess data and (2) call
        subroutines to process different types of data
        """
        #print "CMD",cmd
        global Lineno
        global Filename
        Lineno=Lineno_
        Filename=Filename_

        if __debug__:
           logger.debug("DEBUG: _process_data (start) %s",cmd)
        if len(cmd) == 0:                       #pragma:nocover
           raise ValueError, "ERROR: Empty list passed to Model::_process_data"

        cmd = _preprocess_data(cmd)

        if cmd[0] == "data":
           return True
        if cmd[0] == "end":
           return False
        if cmd[0].startswith('set'):
           _process_set(cmd, _model, _data)
        elif cmd[0].startswith('param'):
           _process_param(cmd, _model, _data, _default)
        elif cmd[0] == 'include':
           _process_include(cmd, _model, _data, _default)
        elif cmd[0] == 'import':
           _process_import(cmd, _model, _data, _default)
        else:
           raise IOError, "ERROR: Unknown data command: "+" ".join(cmd)
        return True



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

import os.path
from TableData import TableData
from pyutilib.component.core import alias
from decimal import Decimal


# format=
# using=
# query=
# user=
# password=
# table=

class db_Table(TableData):

    def __init__(self):
        TableData.__init__(self)


    def open(self):
        if self.filename is None:
            raise IOError, "No data source name was specified"
        if self.filename[0] == '"':
            self.filename = self.filename[1:-1]
        #
        # Initialize self.db
        #
        self.db = None
        if self._data is not None:
            self.db = self._data
        else:
            try:
                self.db = self.connect(self.filename, self.options)
            except Exception:
                raise

    def read(self):
        #
        # Get the table from the database
        #
        if self.db is None:
            return
        cursor = self.db.cursor()
        tmp = []
        if self.options.query is None:
            if self.options.table is None:
                raise IOError, "Must specify 'query' or 'table' option!"
            self.options.query = '"SELECT * FROM %s"' % self.options.table
        if not self.options.table is None:
            for row in cursor.columns(table=self.options.table):
                tmp.append(row.column_name)
            tmp=[tmp]
        else:
            # TODO: extend this logic to create a header row for a SQL query
            pass
        #print "DATA start",self.options.query
        cursor.execute(self.options.query[1:-1])
        for row in cursor:
            #print "DATA",list(row)
            ttmp=[]
            for data in list(row):
                if isinstance(data,Decimal):
                    ttmp.append(float(data))
                elif data is None:
                    ttmp.append('.')
                else:
                    ttmp.append(data)
            tmp.append(ttmp)
        #print 'FINAL',tmp
        #
        # Process data from the table
        #
        if type(tmp) in (int,long,float):
            if not self.options.param is None:
                self._info = ["param",self.options.param,":=",tmp]
            elif len(self.options.symbol_map) == 1:
                self._info = ["param",self.options.symbol_map[self.options.symbol_map.keys()[0]],":=",tmp]
            else:
                raise IOError, "Data looks like a parameter, but multiple parameter names have been specified: %s" % str(self.options.symbol_map)
        elif len(tmp) == 0:
            raise IOError, "Empty range '%s'" % self.options.range
        else:
            self._set_data(tmp[0], tmp[1:])
        return True

    def close(self):
        if self._data is None and not self.db is None:
            del self.db

    def connect(self, connection, options, kwds={}):
        try:
            mod = __import__(options.using)
            args = [connection]
            if not options.user is None:
                args.append(options.user)
            if not options.password is None:
                args.append(options.password)
            #print "XXX",args,self.options
            return mod.connect(*args, **kwds)
        except ImportError:
            return None
        

try:
    import pyodbc

    class pyodbc_db_Table(db_Table):

        alias('pyodbc', "Manage IO with a %s database interface" % 'pyodbc')

        def __init__(self):
            db_Table.__init__(self)
            self.using = 'pyodbc'

        def connect(self, connection, options):
            if not options.driver is None:
                ctype = options.driver
            elif '.' in connection:
                ctype = connection.split('.')[-1]
            connection = self.create_connection_string(ctype, connection, options)
            if ctype in ['xls','excel'] or '.xls' in connection:
                return db_Table.connect(self, connection, options, {'autocommit':True})
            return db_Table.connect(self, connection, options)
        
        def create_connection_string(self, ctype, connection, options):
            if ctype in ['xls', 'excel']:
                return "Driver={Microsoft Excel Driver (*.xls)}; Dbq=%s;" % connection
            if ctype in ['mdb', 'access']:
                return "Driver={Microsoft Access Driver (*.mdb)}; Dbq=%s;" % connection
            return connection
                    
except ImportError:
    pass


for module in ['pymysql']:
    try:
        __import__(module)
        class tmp(db_Table):
            alias(module, "Manage IO with a %s database interface" % module)
            def __init__(self):
                db_Table.__init__(self)
                self.using = module

    except ImportError:
        pass




# same as example 1, but with maximization. should yield an unboundedness error.

from coopr.pyomo import *

model = AbstractModel()

model.X = Var(bounds=(-5,5))

breakpoints = [0]
slopes =      [-1, 1]

model.Z = Piecewise(breakpoints=breakpoints,
                    slopes=slopes,
                    offset=0,
                    value=model.X)

model.obj = Objective(rule = lambda model: model.Z, sense=maximize)




__all__ = ['IndexedComponent']

from component import Component
from sets import _BaseSet, Set, set_options


class IndexedComponent(Component):

    def __init__(self, *args, **kwds):
        #
        # The default ctype is None, which triggers an error in the
        # Component base class.
        #
        component_type = kwds.get('ctype', None)
        Component.__init__(self, component_type)
        
        self._ndim=0
        self._index_set=None
        self._index={}
        if len(args) == 0:
            self._index={None:None}
        elif len(args) == 1:
            if isinstance(args[0],_BaseSet):
                self._index=args[0]
            elif isinstance(args[0],IndexedComponent):
                    raise TypeError, "Cannot index a component with a non-set component"
            else:
                try:
                    options = getattr(args[0],'set_options')
                    options['initialize'] = args[0]
                    self._index=Set(**options)
                except:
                    self._index=Set(initialize=args[0])
        else:
            tmp = []
            for arg in args:
                if isinstance(arg,_BaseSet):
                    tmp.append(arg)
                elif isinstance(arg,IndexedComponent):
                    raise ValueError, "Cannot index a component with a non-set component"
                else:
                    try:
                        options = getattr(arg,'set_options')
                        options['initialize'] = arg
                        tmp.append( Set(**options) )
                    except:
                        tmp.append( Set(initialize=arg) )
            self._index_set=tuple(tmp)
        self._compute_dim()

    def index(self):
        return self._index

    def _compute_dim(self):
        if self._ndim is None or self._ndim is 0:
            if self._index_set is None:
                if type(self._index) is dict:
                    if len(self._index) is 0:
                        self._ndim = 0
                    else:
                        val = self._index[self._index.keys()[0]]
                        if type(val) is tuple:
                            self._ndim = len(val)
                        elif val is None:
                            self._ndim = 0
                        else:
                            self._ndim = 1
                else:
                    self._ndim = self._index.dimen
            else:
                self._ndim = 0
                for iset in self._index_set:
                    self._ndim += iset.dimen





#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Var', 'VarStatus', '_VarBase', "Piecewise"]

import logging
from component import Component
from numvalue import *
from numvalue import create_name
from label import *
from pyutilib.component.core import alias
import types
from set_types import *
from indexed_component import IndexedComponent
import pyutilib.math
import pyutilib.misc
from pyutilib.enum import Enum
import sys
from param import _ParamValue
from coopr.pyomo.base.util import isfunctor

logger = logging.getLogger('coopr.pyomo')

VarStatus = Enum( 'undefined', 'fixed_by_presolve', 'fixed_by_optimizer',
                  'unused', 'used', )

class _VarValue(NumericValue):
    """Holds the numeric value of a variable"""

    __slots__ = ['var','index','id','label','initial','active','lb','ub','fixed','status','_is_binary','_is_integer','_is_continuous','ampl_var_id']


    # IMPT: If a specific argument is intended to be left unassigned, use the following default values:
    # name: None
    # domain: None

    def __init__(self, name, domain):
        """Constructor"""

        # NOTE: the "within" keyword takes precedence over the "domain" keyword.
        NumericValue.__init__(self, name, domain, None, False)

        # NOTE: the "name" attribute (part of the base NumericValue class) is 
        #       typically something like some_var[x,y,z] - an easy-to-read
        #       representation of the variable/index pair.

        # NOTE: both of the following are presently set by the parent. arguably, we
        #       should provide keywords to streamline initialization.
        self.var = None # the "parent" variable.
        self.index = None # the index of this variable within the "parent"

        # TBD: Document where this gets populated.
        self.id = None

        # the label is a modification of the input name, with certain symbols
        # that are typically problematic for solver input files re-mapped (e.g., 
        # curly braces) to more friendly symbols. create a default label right
        # away - it can be modified if a particular solver needs it.
        # self.label = None (is set one way or another via the following method)
        self.sync_label()

        # the default initial value for this variable.
        self.initial = None

        # is this variable an active component of the current model?
        self.active = True

        # IMPORTANT: in contrast to the initial value, the lower and upper
        #             bounds of a variable can in principle be either numeric constants,
        #             parameter values, or expressions - anything is OK, as long as
        #             invoking () is legal.
        self.lb = None
        self.ub = None

        self.fixed = False
        self.status = VarStatus.undefined

        # cached for efficiency purposes - isinstance is not cheap.
        self._is_binary = isinstance(self.domain, BooleanSet)
        self._is_integer = isinstance(self.domain, IntegerSet)
        self._is_continuous = not (self._is_binary or self._is_integer)

        # for NL - really a suffix that should be moved to the Var parent class.
        self.ampl_var_id = None

    def __str__(self):
        # the name can be None, in which case simply return "".
        if self.name is None:
           return ""
        else:
           return self.name

    def __getstate__(self):
       result = NumericValue.__getstate__(self)
       for i in _VarValue.__slots__:
          result[i] = getattr(self, i)
       return result

    def sync_label(self):
        if self.name is not None:
           self.label = label_from_name(self.name)
           if (self.var is not None) and (self.var.model is not None):
              self.var.model._name_varmap[self.label] = self
        else:
           self.label = None

    def getattrvalue(self, attr_name):
       if attr_name not in self.var._attr_values:
          raise RuntimeError,"Variable="+str(self.var.name)+" does not have an attributed defined with name="+str(attr_name)
       return self.var._attr_values[attr_name][self.index]

    def setattrvalue(self, attr_name, attr_value):
       if attr_name not in self.var._attr_values:
          raise RuntimeError,"Variable="+str(self.var.name)+" does not have an attributed defined with name="+str(attr_name)        
       self.var._attr_values[attr_name][self.index] = attr_value

    def activate(self):
        self.active=True

    def deactivate(self):
        self.active=False

    def setlb(self, value):
        # python only has 3 numeric built-in type that we deal with (we ignore
        # complex numbers).
        if value is None:
            self.lb = None
        elif isinstance(value, (int, long, float)):
            self.lb = NumericConstant(None,None,value)
        elif isinstance(value, _ParamValue):
            self.lb = value
        else:
            msg = "Unknown type '%s' supplied as variable lower bound - "     \
                  'legal types are numeric constants or parameters'
            raise ValueError, msg % str( type(value) )

    def setub(self, value):
        # python only has 3 numeric built-in type that we deal with (we ignore
        # complex numbers).
        if isinstance(value, (int, long, float)):
            self.ub = NumericConstant(None,None,value)
        elif value is None:
            self.ub = None
        elif isinstance(value, _ParamValue):
            self.ub = value
        else:
            msg = "Unknown type '%s' supplied as variable lower bound - "     \
                  'legal types are numeric constants or parameters'
            raise ValueError, msg % str( type(value) )


    def fixed_value(self):
        if self.fixed:
            return True
        return False

    def is_constant(self):
        if self.fixed:
            return True
        return False

    def polynomial_degree(self):
        if self.fixed:
            return 0
        return 1

    def is_binary(self):
        return self._is_binary

    def is_integer(self):
        return self._is_integer

    def is_continuous(self):
        return self._is_continuous

    def pprint(self, ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, str(self),

    def _tighten_bounds(self):
        """
        Attempts to tighten the lower and upper bounds on 
        this variable by using the domain.
        """
        try:
            dlb, dub = self.domain.bounds()
        except:
            # No explicit domain bounds
            return

        if (dlb is not None):
           if (self.lb is None) or (dlb > self.lb.value):
              self.setlb(dlb)

        if (dub is not None):
           if (self.ub is None) or (dub < self.ub.value):
              self.setub(dub)


#
# Variable attributes:
#
# Single variable:
#
# x = Var()
# x.fixed = True
# x.domain = Reals
# x.name = "x"
# x.setlb(-1.0)
# x.setub(1.0)
# x.initial = 0.0
# x = -0.4
#
# Array of variables:
#
# y = Var(set1,set2,...,setn)
# y.fixed = True (fixes all variables)
# y.domain = Reals
# y.name = "y"
# y[i,j,...,k] (returns value of a given index)
#
# y[i,j,...,k].fixed
# y[i,j,...,k].lb()
# y[i,j,...,k].ub()
# y[i,j,...,k].initial
#

class _VarBase(IndexedComponent):
    """A numeric variable, which may be defined over a index"""

    """ Constructor
        Arguments:
           name         The name of this variable
           index        The index set that defines the distinct variables.
                          By default, this is None, indicating that there
                          is a single variable.
           domain       A set that defines the type of values that
                          each parameter must be.
           default      A set that defines default values for this
                          variable.
           bounds       A rule for defining bounds values for this
                          variable.
           rule         A rule for setting up this parameter with
                          existing model data
    """
    # TODO: default and rule keywords are not used?  Need to talk to Bill ...?
    def __init__(self, *args, **kwd):
        tkwd = {'ctype':Var}
        IndexedComponent.__init__(self, *args, **tkwd)

        # Default keyword values
        #
        self._attr_declarations = {} # maps the name of an attribute (i.e., a suffix) to its default value.
        self._attr_values = {} # maps the name of an attribute to a dictionary containing (index, value) pairs.
        self._varval = {}
        self._initialize = kwd.pop('initialize', None )
        self.name   = kwd.pop('name', 'unknown')
        self.domain = kwd.pop('within', Reals )
        self.domain = kwd.pop('domain', self.domain )
        self.bounds = kwd.pop('bounds', None )
        self.doc    = kwd.pop('doc', None )

        self._binary_keys = []
        self._continuous_keys = []
        self._integer_keys = []

        # Check for domain rules
        if isfunctor(self.domain):
            self._domain_rule = self.domain

            # Individual variables will be restricted
            self.domain = None
        else:
            self._domain_rule = None

        if kwd:
            msg = "Var constructor: unknown keyword '%s'"
            raise ValueError, msg % kwd.keys()[0]

    def as_numeric(self):
        if None in self._varval:
            return self._varval[None]
        return self

    def is_indexed(self):
        return self._ndim > 0

    def is_expression(self):
        return False

    def is_relational(self):
        return False

    def keys(self):
        return self._varval.keys()

    def binary_keys(self):
        """ Returns the keys of all binary variables """
        return self._binary_keys

    def continuous_keys(self):
        """ Returns the keys of all continuous variables """
        return self._continuous_keys

    def integer_keys(self):
        """ Returns the keys of all integer variables """
        return self._integer_keys

    def __iter__(self):
        return self._varval.keys().__iter__()

    def __contains__(self,ndx):
        return ndx in self._varval

    def dim(self):
        return self._ndim

    def reset(self):
        for value in self._varval.itervalues():
            value.set_value(value.initial)

    def __len__(self):
        return len(self._varval)

    def __setitem__(self,ndx,val):
        #print "HERE",ndx,val, self._valid_value(val,False), self.domain
        if (None in self._index) and (ndx is not None):
            # allow for None indexing if this is truly a singleton
            msg = "Cannot set an array value in singleton variable '%s'"
            raise KeyError, msg % self.name

        if ndx not in self._index:
            msg = "Cannot set the value of array variable '%s' with invalid " \
                  "index '%s'"
            raise KeyError, msg % ( self.name, str(ndx) )

        if not self._valid_value(val,False):
            msg = "Cannot set variable '%s[%s]' with invalid value: %s"
            raise ValueError, msg % ( self.name, str(ndx), str(val) )
        self._varval[ndx].value = val

    def __getitem__(self,ndx):
        """This method returns a _VarValue object.  This object can be
           coerced to a numeric value using the value() function, or using
           explicity coercion with float().
        """
        try:
            return self._varval[ndx]
        except KeyError: # thrown if the supplied index is hashable, but not defined.
            msg = "Unknown index '%s' in variable %s;" % (str(ndx), self.name)
            if (isinstance(ndx, (tuple, list)) and len(ndx) != self.dim()):
                msg += "    Expecting %i-dimensional indices" % self.dim()
            else:
                msg += "    Make sure the correct index sets were used.\n"
                msg += "    Is the ordering of the indices correct?"
            raise KeyError, msg
        except TypeError, msg: # thrown if the supplied index is not hashable
            msg2 = "Unable to index variable %s using supplied index with " % self.name
            msg2 += str(msg)
            raise TypeError, msg2

    def _add_domain_key(self, ndx, domain):
        """ Register an index with a specific set of keys """
        if isinstance(domain, BooleanSet):
            self._binary_keys.append(ndx)
        elif isinstance(domain, IntegerSet):
            self._integer_keys.append(ndx)
        else:
            self._continuous_keys.append(ndx)

    def construct(self, data=None):
        if __debug__:   #pragma:nocover
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Variable, name=%s, from data=%s", self.name, `data`)
        if self._constructed:
            return
        self._constructed=True
        #
        # Construct _VarValue objects for all index values
        #
        if self._ndim > 0:
            if type(self._initialize) is dict:
                self._index = self._initialize.keys()

            # cached for efficiency
            self_name = self.name 
            self_domain_rule = self._domain_rule
            self_domain = self.domain

            for ndx in self._index:
                # Check for domain rules
                if self_domain_rule is not None:
                    domain = self_domain_rule(ndx, self.model)
                    if isinstance(domain, BooleanSet):
                        self._binary_keys.append(ndx)
                    elif isinstance(domain, IntegerSet):
                        self._integer_keys.append(ndx)
                    else:
                        self._continuous_keys.append(ndx)
                else:
                    domain = self_domain

                new_varval = _VarValue(create_name(self_name,ndx),
                                                   domain)
                new_varval.var = self
                new_varval.index = ndx

                self._varval[ndx] = new_varval

        else:
           # if the dimension is a singleton (i.e., we're dealing
           # with a _VarElement), then the name was set after
           # the VarValue construction for the singleton - and as
           # a consequence, the label attribute has yet to be defined.
           # so set it! 
           if self.name is not None:
              self._varval[None].label = label_from_name(self.name)

        #
        # Define the _XXX_keys objects if domain isn't a rule;
        # they were defined individually above in the case of
        # a rule
        #
        if not self._domain_rule is not None:
            if isinstance(self.domain, BooleanSet):
                self._binary_keys = self._varval.keys()
            elif isinstance(self.domain, IntegerSet):
                self._integer_keys = self._varval.keys()
            else:
                self._continuous_keys = self._varval.keys()

        #
        # Initialize values with a dictionary if provided
        #
        if self._initialize is not None:
            #
            # Initialize values with the _rule function if provided
            #
            if self._initialize.__class__ is types.FunctionType:
                for key in self._varval:
                    if key is None:
                        tmp = []
                    elif key.__class__ is tuple:
                        tmp = list(key)
                    else:
                        tmp = [key]
                    tmp.append(self.model)
                    val = self._initialize(*tuple(tmp))
                    self._valid_value(val, True)
                    self._varval[key].value = self._varval[key].initial = val
            elif self._initialize.__class__ is dict:
                for key in self._initialize:
                    val = self._initialize[key]
                    self._valid_value(val, True)
                    self._varval[key].value = self._varval[key].initial = val
            else:
                for key in self._index:
                    val = self._initialize
                    self._valid_value(val, True)
                    self._varval[key].value = self._varval[key].initial = val
        #
        # Initialize bounds with the bounds function if provided
        #
        if self.bounds is not None:

           if type(self.bounds) is tuple:

              # bounds are specified via a tuple - same lower and upper bounds for all var values!

              (lb, ub) = self.bounds
              
              # do some simple validation that the bounds are actually finite - otherwise, set them to None.        
              if (lb is not None) and (not pyutilib.math.is_finite(value(lb))):
                 lb = None
              if (ub is not None) and (not pyutilib.math.is_finite(value(ub))):
                 ub = None

              for key,varval in self._varval.iteritems():
                 if lb is not None:
                    varval.setlb(lb)
                 if ub is not None:
                    varval.setub(ub)

           else:

             # bounds are specified via a function

             for key,varval in self._varval.iteritems():
                 if isinstance(key,tuple):
                     tmp = list(key)
                 elif key is None:
                     tmp = []
                 else:
                     tmp = [key]
                 tmp.append(self.model)
                 tmp = tuple(tmp)
                 (lb, ub) = self.bounds(*tmp)
                 varval.setlb(lb)
                 varval.setub(ub)

             for key in self._varval:
                 if self._varval[key].lb is not None and not pyutilib.math.is_finite(self._varval[key].lb()):
                     self._varval[key].setlb(None)
                 if self._varval[key].ub is not None and not pyutilib.math.is_finite(self._varval[key].ub()):
                     self._varval[key].setub(None)
        #
        # Iterate through all variables, and tighten the bounds based on
        # the domain bounds information.
        #
        # Only done if self.domain is not a rule. If it is, _VarArray level
        # bounds become meaningless, since the individual _VarElement objects
        # likely have more restricted domains.
        #
        if self._domain_rule is None:
            dbounds = self.domain.bounds()
            if not dbounds is None and dbounds != (None,None):
                for key in self._varval:
                    if not dbounds[0] is None:
                        if self._varval[key].lb is None or                     \
                            dbounds[0] > self._varval[key].lb():
                            self._varval[key].setlb(dbounds[0])

                    if not dbounds[1] is None:
                        if self._varval[key].ub is None or                     \
                            dbounds[1] < self._varval[key].ub():
                            self._varval[key].setub(dbounds[1])
        #
        # Setup declared attributes for all variables
        #
        for attr_name in self._attr_declarations:
            default_value = self._attr_declarations[attr_name][0]
            suffix_dict = {}
            for key in self._varval:
                suffix_dict[key]=default_value
            self._attr_values[attr_name] = suffix_dict
        #
        # update the parent model (if any) name->var map.
        #
        if self.model is not None:
           for varval in self._varval.itervalues():
              self.model._name_varmap[varval.label] = varval

    def pprint(self, ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, "  ",self.name,":",
        print >>ostream, "\tSize="+str(len(self)),
        if self.domain is not None:
            print >>ostream, "\tDomain="+self.domain.name
        else:
            print >>ostream, "\tDomain=None"
        if self._index_set is not None:
            print >>ostream, "\tIndicies: ",
            for idx in self._index_set:
               print >>ostream, str(idx.name)+", ",
            print ""
        if None in self._varval:
           print >>ostream, "\tInitial Value : Lower Bound : Upper Bound : "  \
                            "Current Value: Fixed: Status"
           lb_value = None
           if self._varval[None].lb is not None:
              lb_value = self._varval[None].lb()
           ub_value = None
           if self._varval[None].ub is not None:
              ub_value = self._varval[None].ub()

           print >>ostream, "\t %s : %s : %s : %s : %s : %s" % (
             str(self._varval[None].initial),
             str(lb_value),
             str(ub_value),
             str(self._varval[None].value),
             str(self._varval[None].fixed),
             str(self._varval[None].status)
           )
        else:
           print >>ostream, "\tKey : Initial Value : Lower Bound : "          \
                            "Upper Bound : Current Value: Fixed: Status"
           tmp=self._varval.keys()
           tmp.sort()
           for key in tmp:
             initial_val = self._varval[key].initial
             lb_value = None
             if self._varval[key].lb is not None:
                lb_value = self._varval[key].lb()
             ub_value = None
             if self._varval[key].ub is not None:
                ub_value = self._varval[key].ub()

             print >>ostream, "\t%s : %s : %s : %s : %s : %s : %s" % (
               str(key),
               str(initial_val),
               str(lb_value),
               str(ub_value),
               str(value(self._varval[key].value)),
               str(value(self._varval[key].fixed)),
               str(self._varval[key].status)
             )


    def display(self, prefix="", ostream=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, prefix+"Variable "+self.name,":",
        print >>ostream, "  Size="+str(len(self)),
        print >>ostream, "Domain="+self.domain.name
        if None in self._varval:
           print >>ostream, "%s  Value=%s" % (
             prefix,
             pyutilib.misc.format_io(self._varval[None].value)
           )
        else:
           for key in self._varval:
             val = self._varval[key].value
             print >>ostream, prefix+"  "+str(key)+" : "+str(val)

    def declare_attribute(self, name, default=None):
        """
        Declare a user-defined attribute.
        """
        if name[0] == "_":
            msg = "Cannot define an attribute that begins with  '_'"
            raise AttributeError, msg
        if name in self._attr_declarations:
            raise AttributeError, "Attribute %s is already defined" % name
        self._attr_declarations[name] = (default,)
        #if not default is None:
            #self._valid_value(default)
        #
        # If this variable has been constructed, then
        # generate this attribute for all variables
        #
        # add to self._attr_values
        if len(self._varval) > 0:
            suffix_dict = {}
            for key in self._varval:
                suffix_dict[key]=default
            self._attr_values[name] = suffix_dict

    def attribute_defined(self, name):
        """
        Determine if a user-defined attribute actually exists!        
        """
        return name in self._attr_declarations

# a _VarElement is the implementation representing a "singleton" or non-indexed variable.
# NOTE: this class derives from both a "slot"ized base class (_VarValue) and a normal
#       class with a dictionary (_VarBase) - beware (although I believe the __getstate__
#       implementation should be attribute-independent).

class _VarElement(_VarBase, _VarValue):

    def __init__(self, *args, **kwd):

        _VarValue.__init__(self, \
                           kwd.get('name', None), \
                           kwd.get('within', kwd.get('domain', Reals)))
        _VarBase.__init__(self, *args, **kwd)
        self._varval[None] = self
        self._varval[None].var = self
        self._varval[None].index = None

    def __call__(self, exception=True):
        if None in self._varval:
            return self._varval[None].value
        return None

    def __getstate__(self):
       result = _VarValue.__getstate__(self)
       for key,value in self.__dict__.iteritems():
          result[key]=value
       return result

    def is_constant(self):
       return _VarValue.is_constant(self)

# a _VarArray is the implementation representing an indexed variable.

class _VarArray(_VarBase):

    def __init__(self, *args, **kwds):

        _VarBase.__init__(self, *args, **kwds)
        self._dummy_val = _VarValue(kwds.get('name', None), \
                                    kwds.get('within', kwds.get('domain', Reals)))

    def __float__(self):
        raise TypeError, "Cannot access the value of array variable "+self.name

    def __int__(self):
        raise TypeError, "Cannot access the value of array variable "+self.name

    def _valid_value(self,value,use_exception=True):
        #print "VALID",self._dummy_val.domain
        return self._dummy_val._valid_value(value,use_exception)

    def set_value(self, value):
        msg = "Cannot specify the value of array variable '%s'"
        raise ValueError, msg % self.name

    def __str__(self):
        return self.name

    def construct(self, data=None):
        _VarBase.construct(self, data)

        for (ndx, var) in self._varval.iteritems():
            var._tighten_bounds()


class Var(Component):
    """
    Variable objects that are used to construct Pyomo models.
    """

    alias("Var", "Decision variables in a model.")

    def __new__(cls, *args, **kwds):
        if args == ():
            self = _VarElement(*args, **kwds)
        else:
            self = _VarArray(*args, **kwds)
        return self


class Piecewise(_VarArray):
    """
    A piecewise linear expression.

    Usage:

    model.X = Var() # pre-requisite

    # Let breakpoint_rule and slope_rule define a piecewise linear
    # function f(x) when indexed over sets (...)
    #
    # Then model.XF is a variable constrained by model.XF = f(model.X)
    model.XF = Piecewise(
                         breakpoint_rule=breakPointRule,
                         slope_rule=slopeRule,
                         offset=0,
                         value=model.X,
                         forcesos2=True
                         )

    # Evaluate the piecewise expression at 7.0
    model.C2.eval(7.0)

    Breakpoints refer to the values on the domain that separate the
    domain into distinct intervals. All intervals are assumed to be
    closed on the lower bound and open on the upper bound, with the
    exception of the interval extending to negative infinity on the
    lower bound. The first slope specified is assumed to be the slope
    of an interval coming from negative infinity to the first
    breakpoint; all other slopes are assumed to correspond to the
    intervals between the current and next breakpoint. The offset
    keyword is used to indicate the y-intercept of the expression
    (default is zero).

    Example:

    The concave function

           / 2x + 3, x < 0
    f(x) = | 3,      0 < x < 1
           \ 3 - x,  1 < x

    would be modeled as:

    breakpoints = [0, 1]
    slopes = [2, 0, -1]
    offset = 3

    If the function should have a bounded domain, specify more
    breakpoints than slopes.

    Example:

    The concave function

           / 2x + 3, -2 < x < 0
    f(x) = | 3,       0 < x < 1
           \ 3 - x,   1 < x < 4

    would be modeled as:

    breakpoints = [-2, 0, 1, 4]
    slopes = [2, 0, -1]
    offset = 3

    If we model a bounded domain, the variable passed to model.value
    will be bounded by this domain. Finally, for bounded domains, the
    offset is the vertical offset of the leftmost breakpoint.

    The forcesos2 keyword is simply used to always generate the sos2
    transformation, even if the slopes specify a concave/convex function.

    """

    alias("Piecewise", "A variable constrained by a piecewise function.")

    def __init__(self, *args, **kwargs):
        """
        Create a new Piecewise variable
        """

        # ---------------------------------------------------------------------
        # Process arguments

        # We can either get explicit breakpoints and slopes, or rules
        # defining them. 
        self._breakpoints = kwargs.pop('breakpoints', None)
        self._slopes = kwargs.pop('slopes', None)

        self._breakpoint_rule = kwargs.pop('breakpoint_rule', None)
        self._slope_rule = kwargs.pop('slope_rule', None)

        self._forcesos2 = kwargs.pop('forcesos2', False)

        # The corresponding domain variable.
        self.variable = kwargs.pop('value', None)

        # The y-intercept of the entire piecewise construct.
        self.offset = kwargs.pop('offset', 0)

        # Don't pop the name, we want to pass this to superclass constructor
        tmpname = kwargs.get('name', 'unknown')

        # ---------------------------------------------------------------------
        # Error checking

        # Make sure a domain variable was passed.
        if self.variable is None:
            raise TypeError, "Specify a Variable object for the 'value' " \
                  "attribute for Piecewise object '%s'" % tmpname

        # Make sure repetitive arguments aren't specified
        if self._breakpoints is not None and self._breakpoint_rule is not None:
            raise TypeError, "Specify only one of 'breakpoints' and " \
                  "'breakpointrule' in Piecewise object '%s'" % tmpname
        if self._slopes is not None and self._slope_rule is not None:
            raise TypeError, "Specify only one of 'slopes' and " \
                  "'sloperule' in Piecewise object '%s'" % tmpname

        # Make sure enough arguments were specified
        if self._breakpoints is None and self._breakpoint_rule is None:
            raise TypeError, "Specify exactly one of 'breakpoints' or " \
                  "'breakpointrule' in Piecewise object '%s'" % tmpname
        if self._slopes is None and self._slope_rule is None:
            raise TypeError, "Specify exactly one of 'slopes' or " \
                  "'sloperule' in Piecewise object '%s'" % tmpname

        # ---------------------------------------------------------------------
        # Initialization

        # NOTE: The following is a bit confusing. _VarElement is in turn derived from _VarBase,
        #       which is in turn derived from IndexedComponent. if the component really isn't
        #       indexed, then self._ndim will equal 0. 
        # 
        _VarArray.__init__(self, *args, **kwargs)

        # If index sets were passed to either breakpoint or slopes,
        # make sure the appropriate rule is defined.
        if (self._ndim > 0) and (self._breakpoint_rule is None):
            raise TypeError, "Cannot specify an indexed Piecewise variable without " \
                  "'breakpoint_rule': object='%s'" % tmpname
        if (self._ndim > 0) and (self._slope_rule is None):
            raise TypeError, "Cannot specify an indexed Piecewise variable without " \
                  "'slope_rule': object='%s'" % tmpname

    def construct(self, data=None):
        """
        Construct the breakpoint and slope expressions
        """

        if __debug__:
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Piecewise component, name=%s, from data=%s", self.name, `data`)

        # take care of the base variable array first.
        _VarArray.construct(self, data)           

        # NOTE: the _breakvals and _slopevals attributes can be either lists (if the 
        #       component is not indexed) or list of lists (if the component is indexed).

        # Construct the breakpoints
        self._breakvals = list()
        if self._breakpoints is not None:
            # Unpack breakpoints from an iterable
            for exp in self._breakpoints:
                if exp is not None:
                    self._breakvals.append(exp)
        else:
            # Get breakpoints from a rule
            if (self._ndim == 0):
                # Rule isn't indexed
                exp = self._breakpoint_rule((self.model,))
                if not isinstance(exp, (list, tuple)):
                   raise TypeError, "A breakpoint rule is expected to return a list or a tuple - returned type was "+str(type(exp))
                if exp is not None:
                    self._breakvals = exp
            else:
                # component is indexed
                for index in self._index:
                    # Make index a list, and then append self.model
                    if isinstance(index, (list, tuple)):
                        index = list(index)
                    else:
                        index = [index]
                    index.append(self.model)

                    # Get the next breakpoint
                    exp = self._breakpoint_rule(*index)
                    if exp is not None:
                        self._breakvals.append(exp)

        # Construct the slopes
        self._slopevals = list()
        if self._slopes is not None:
            # Unpack slopes from an iterable
            for exp in self._slopes:
                if exp is not None:
                    self._slopevals.append(exp)
        else:
            # Get slopes from a rule
            if (self._ndim == 0):
                # Rule isn't indexed
                exp = self._slope_rule((self.model,))
                if not isinstance(exp, (list, tuple)):
                   raise TypeError, "A slope rule is expected to return a list or a tuple - returned type was "+str(type(exp))
                if exp is not None:
                    self._slopevals = exp
            else:
                # component is indexed
                for index in self._index:
                    # Make index a list, and then append self.model
                    if isinstance(index, (list, tuple)):
                        index = list(index)
                    else:
                        index = [index]
                    index.append(self.model)

                    # Get the next breakpoint
                    exp = self._slope_rule(*index)
                    if exp is not None:
                        self._slopevals.append(exp)

        if __debug__:
            if logger.isEnabledFor(logging.DEBUG):
                if (self._ndim == 0):
                    logger.debug("Slopes="+str(self._slopevals))
                else:
                    i = 0
                    for index in self._index:
                        logger.debug("Slope["+str(index)+"]="+str(self._slopevals[i]))
                        i += 1

                if (self._ndim == 0):
                    logger.debug("Breakpoints="+str(self._breakvals))
                else:
                    i = 0
                    for index in self._index:
                        logger.debug("Breakpoint["+str(index)+"]="+str(self._breakvals[i]))
                        i += 1

        # If N breakpoints are specified, then there can be N-1 or N+1 slopes.
        if (self._ndim == 0):
           if abs(len(self._slopevals) - len(self._breakvals)) != 1:
               msg = "Error constructing Piecewise object '%(name)s'; "          \
                     "%(breaks)i breakpoints and %(slopes)i slopes were "        \
                     "specified.\n\tGiven %(breaks)i breakpoints, there should " \
                     "be %(breaks)i-1 or %(breaks)i+1 slopes."
               raise ValueError, msg % {
                 "breaks" : len(self._breakvals),
                 "slopes" : len(self._slopevals),
                 "name"   : self.name
               }
        else:
           for i in range(0,len(self._breakvals)):
              if abs(len(self._slopevals[i]) - len(self._breakvals[i])) != 1:
                  msg = "Error constructing Piecewise object '%(name)s'; "          \
                        "%(breaks)i breakpoints and %(slopes)i slopes were "        \
                        "specified.\n\tGiven %(breaks)i breakpoints, there should " \
                        "be %(breaks)i-1 or %(breaks)i+1 slopes."
                  raise ValueError, msg % {
                    "breaks" : len(self._breakvals[i]),
                    "slopes" : len(self._slopevals[i]),
                    "name"   : self.name
                  }

        # If a user specified an unbounded problem, add "None" to both ends of self._breakvals.
        if (self._ndim == 0):
           if len(self._breakvals) < len(self._slopevals):
              self._breakvals.insert(0, None)
              self._breakvals.append(None)
        else:
           for i in range(0,len(self._breakvals)):
              if len(self._breakvals[i]) < len(self._slopevals[i]):
                 self._breakvals[i].insert(0, None)
                 self._breakvals[i].append(None)

        # Finally, generate the constraints for each index (if applicable).
        if (self._ndim == 0):
           self._generate_constraints(None, self._breakvals, self._slopevals)
        else:
           i = 0
           for index in self._index:
              self._generate_constraints(index, self._breakvals[i], self._slopevals[i])
              i += 1


    def _concave_or_convex(self):
        """
        Determines if the expressions forms a concave or convex expression.
        Returns -1 if concave, 1 if convex, 2 if affine, and 0 otherwise.
        """

        # Must be convex if there are less than three segments
        if len(self._slopevals) <= 2:
            if len(self._slopevals) == 1:
                # Affine
                return 2
            elif self._slopevals[0] < self._slopevals[1]:
                # Convex
                return 1
            elif self._slopevals[0] == self._slopevals[1]:
                # Affine
                return 2
            else:
                # Concave
                return -1

        # Find first segment that doesn't have the same slope,
        # get initial direction
        start = 1
        while start < len(self._slopevals):
            if value(self._slopevals[start]) == \
                   value(self._slopevals[start-1]):
                start += 1
            else:
                direction = value(self._slopevals[start]) > \
                            value(self._slopevals[start-1])
                break

        # If we didn't find a change in slope, we're affine
        if start == len(self._slopevals) - 1:
            # Affine
            return 2

        # Make sure the rest of the segments follow the same direction
        for i in xrange(start, len(self._slopevals)):
            if not (direction and value(self._slopevals[i]) >= \
                    value(self._slopevals[i-1])):
                return 0

        if direction:
            # Convex
            return 1
        else:
            # Concave
            return -1

    def is_convex(self):
        res = self._concave_or_convex()
        return res == 1 or res == 2

    def is_concave(self):
        res = self._concave_or_convex()
        return res == -1 or res == 2

    def is_affine(self):
        return self._concave_or_convex() == 2

    def is_concave_or_convex(self):
        return self._concave_or_convex() != 0

    def _generate_constraints(self, index, breakvals, slopevals):
        """
        Generate the constraints on the variable. If this is a convex
        (or concave) expression, then we simply minimize (or maximize)
        over a number of linear constraints. If not, we introduce SOS2
        variables to constrain the expression.
        """

        # Error if imported at module level
        from constraint import Constraint, SOSConstraint
        from rangeset import RangeSet

        # The root of all implicit constraint, variable, and set names
        baseName = str(self.name) + "[" + str(index) + "]" + "_implicit_"

        # Make a partial closure that returns a rule
        # Arguments passed become closures
        def makeRule(index, slope, hOffset, vOffset):
            expr = slope*(self.variable[index] - hOffset) + vOffset
            if convex:
                def rule(model):
                    return self[index] >= expr
            else:
                def rule(model):
                    return self[index] <= expr
            return rule

        def makeBoundRule(index, value, greater=True):
            if greater:
                def rule(model):
                    return self.variable[index] >= value
            else:
                def rule(model):
                    return self.variable[index] <= value
            return rule

        # Bound model.value if domain is bounded
        if (breakvals[0] is not None):
            self.model.__setattr__(
                baseName + "value_lower_bound",
                Constraint(rule=makeBoundRule(index, breakvals[0], True))
                )
        if (breakvals[-1] is not None):
            self.model.__setattr__(
                baseName + "value_upper_bound",
                Constraint(rule=makeBoundRule(index, breakvals[-1], False))
                )

        convex = self.is_convex()
        concave = self.is_concave()
        if (convex or concave) and (self._forcesos2 is False):
            
            # Maximize (convex) or minimize (concave) over all
            # constraints We first find the interval containing zero

            # Handle the case where the domain has no lower bound.
            if breakvals[0] is None:
                start = 1

                # Line segment with a lower domain bound of -inf
                self.model.__setattr__(
                    baseName + "constraint[%i]" % 0,
                    Constraint(rule=makeRule(index,
                                             slopevals[0],
                                             breakvals[1],
                                             self.offset
                                             ))
                    )
            else:
                start = 0

            # Create the rest of the constraints. Walk from the lower
            # to upper bounds of the domain, keeping track of the
            # vertical offset.
            offset = self.offset
            i = start
            stop = len(breakvals)-1
            if (breakvals[-1] is None):
                stop -= 1

            while i <= stop:
                slope = slopevals[i]
                # Make constraint
                self.model.__setattr__(
                    baseName + "constraint[%i]" % i,
                    Constraint(rule=makeRule(index,
                                             slope,
                                             breakvals[i],
                                             offset))
                    )

                # Update offset
                if i < stop:
                    offset += slope*(breakvals[i+1] - breakvals[i])

                i += 1

        else:
            # Introduce SOS2 constraints

            # IMPORTANT
            #
            # For now, we assume that the domain is bounded. It will
            # fail otherwise. Support for extended domains coming
            # soon.

            # Hack to artificially bound the problem, if necessary
            if breakvals[0] is None:
                breakvals = breakvals[1:]
                slopevals = slopevals[1:]
            if breakvals[-1] is None:
                breakvals = breakvals[:-1]
                slopevals = slopevals[:-1]

            # Walk the breakpoints of the function and find the associated
            # images
            images = [self.offset]
            i = 0
            stop = len(slopevals) - 1
            while i <= stop:
                images.append(
                    images[-1] + slopevals[i] * \
                    (breakvals[i+1] - breakvals[i])
                    )
                i += 1

            # Make the new set indexing the variables
            setName = baseName + "SOS_index_set"
            self.model.__setattr__(
                setName,
                RangeSet(0, len(breakvals)-1)
                )

            # Make the new variables
            varName = baseName + "SOS_variables"
            self.model.__setattr__(
                varName,
                Var(self.model.__getattribute__(setName))
                )

            # Constrain variables

            # It would be nice to just bound all the SOS variables in the
            # Var() class, but then if the variable doesn't appear anywhere
            # else in the LP file CPLEX fails, so we explicity constrain
            # them here

            def makeBoundsRule(varName):
                def boundsRule(i, model):
                    return (0, model.__getattribute__(varName)[i], 1)
                return boundsRule

            self.model.__setattr__(
                baseName + "bounds_constraint",
                Constraint(self.model.__getattribute__(setName),
                           rule=makeBoundsRule(varName))
                )

            # The SOS2 variables sum to 1
            def makeSumRule(varName, setName):
                def sumRule(model):
                    sum(model.__getattribute__(varName)[i] for i in \
                        model.__getattribute__(setName)) == 1
                return sumRule

            self.model.__setattr__(
                baseName + "sum_constraint",
                Constraint(rule=makeSumRule(varName, setName)))

            # The variable self is a convex combination of the images

            def makeImageConvexCombRule(varName, setName, images):
                def imageConvexCombRule(model):
                    return self == sum(
                        model.__getattribute__(varName)[i] * images[i] \
                        for i in model.__getattribute__(setName))

                return imageConvexCombRule

            self.model.__setattr__(
                baseName + "image_convex_combination_constraint",
                Constraint(rule=makeImageConvexCombRule(varName,
                                                        setName,
                                                        images))
                )

            # The variable model.value is a convex combination of the
            # breakpoints

            def makeDomainConvexCombRule(index, varName, setName, breakpoints):
                def domainConvexCombRule(model):
                    return self.variable[index] == sum(
                        model.__getattribute__(varName)[i] * breakpoints[i] \
                        for i in model.__getattribute__(setName))

                return domainConvexCombRule

            self.model.__setattr__(
                baseName + "domain_convex_combination_constraint",
                Constraint(rule=makeDomainConvexCombRule(index,
                                                         varName,
                                                         setName,
                                                         breakvals
                                                         )))

            # Finally, nonzero auxialliary variables must be adjacent
            self.model.__setattr__(
                baseName + "SOS2_constraint",
                SOSConstraint(var=self.model.__getattribute__(varName),
                              set=self.model.__getattribute__(setName),
                              sos=2))




#
# Unit Tests for Param() Objects
#
# PyomoModel                Base test class
# SimpleParam                Test singleton parameter
# ArrayParam1                Test arrays of parameters
# ArrayParam2                Test arrays of parameter with explicit zero default
# ArrayParam3                Test arrays of parameter with nonzero default
# TestIO                Test initialization from an AMPL *.dat file
#

import pyutilib.th as unittest
import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
from coopr.pyomo import *

class PyomoModel(unittest.TestCase):

    def config_repn(self):
        self.repn = 'pyomo_dict'

    def setUp(self):
        self.config_repn()
        self.model = AbstractModel()

    def construct(self,filename):
        self.instance = self.model.create(filename)


class SimpleParam(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        self.model.A = Param(initialize=3.3, repn=self.repn)
        self.instance = self.model.create()

    def tearDown(self):
        if os.path.exists("param.dat"):
           os.remove("param.dat")

    def test_value(self):
        """Check the value of the parameter"""
        tmp = value(self.instance.A)
        self.assertEqual( type(tmp), float)
        self.assertEqual( tmp, 3.3 )
        tmp = float(self.instance.A)
        self.assertEqual( type(tmp), float)
        self.assertEqual( tmp, 3.3 )
        tmp = int(self.instance.A)
        self.assertEqual( type(tmp), int)
        self.assertEqual( tmp, 3 )

    def test_getattr(self):
        """Check the use of the __getattr__ method"""
        if self.instance.components.A.repn_type == 'pyomo_dict':
            self.assertEqual( self.instance.A.value, 3.3)
        else:
            self.assertEqual( self.instance.A, 3.3)

    def test_setattr_value(self):
        """Check the use of the __setattr__ method"""
        self.instance.A = 3.3
        self.assertEqual( self.instance.A, 3.3)
        self.assertEqual( self.instance.components.A[None], 3.3)
        if self.instance.components.A.repn_type == 'pyomo_dict':
            self.assertEqual( self.instance.A.value, 3.3)
        if self.instance.components.A.repn_type == 'pyomo_dict':
            self.instance.A.value = 4.3
            self.assertEqual( self.instance.A.value, 4.3)
            try:
                self.instance.A.value = 'A'
            except ValueError:
                self.fail("fail test_setattr_value")
            else:
                #
                # NOTE: we can set bad values into a NumericValue object
                #
                pass

    def test_setattr_default(self):
        """Check the use of the __setattr__ method"""
        self.model.A = Param(repn=self.repn, default=4.3)
        self.instance = self.model.create()
        self.assertEqual( value(self.instance.A), 4.3)
        self.assertEqual( self.instance.components.A[None], 4.3)

    def test_dim(self):
        """Check the use of dim"""
        self.assertEqual( self.instance.components.A.dim(), 0)

    def test_keys(self):
        """Check the use of keys"""
        self.assertEqual( len(self.instance.components.A.keys()), 1)

    def test_len(self):
        """Check the use of len"""
        self.assertEqual( len(self.instance.components.A), 1)

    def test_getitem(self):
        import coopr.pyomo.base.param
        #print 'xxx',self.instance.components.components()
        #print 'XXX',type(self.instance.A),self.instance.A,self.instance.components.A,'YY'
        #print 'yyy',self.model.A.repn
        #print 'YYY', type(self.instance.A[None])
        if self.instance.components.A.repn_type == 'pyomo_dict':
            self.assertTrue(isinstance(self.instance.A[None], coopr.pyomo.base.param._ParamValue))
        self.assertEqual( self.instance.components.A[None], 3.3)

    def test_check_values(self):
        """Check the use of check_values"""
        self.instance.components.A.check_values()


class MSimpleParam(SimpleParam):

    def config_repn(self):
        self.repn = 'sparse_dict'

MSimpleParam = unittest.skipIf(sys.version_info[:2] < (2,6), "Skipping tests because sparse_dict repn is not supported")(MSimpleParam)


class ArrayParam1(SimpleParam):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        self.model.Z = Set(initialize=[1,3])
        self.model.A = Param(self.model.Z, initialize={1:1.3}, repn=self.repn)
        self.instance = self.model.create()

    def test_value(self):
        try:
          tmp = value(self.instance.A)
        except ValueError:
          pass
        else:
          self.fail("test_value")
        try:
          tmp = float(self.instance.A)
        except TypeError:
          pass
        except ValueError:
          pass
        else:
          self.fail("test_value")
        try:
          tmp = int(self.instance.A)
        except TypeError:
          pass
        except ValueError:
          pass
        else:
          self.fail("test_value")
        tmp = value(self.instance.A[1])
        self.assertEqual( type(tmp), float)
        self.assertEqual( tmp, 1.3 )
        tmp = float(self.instance.A[1])
        self.assertEqual( type(tmp), float)
        self.assertEqual( tmp, 1.3 )
        tmp = int(self.instance.A[1])
        self.assertEqual( type(tmp), int)
        self.assertEqual( tmp, 1 )

    def test_call(self):
        """Check the use of the __call__ method"""
        try:
          tmp = self.instance.A()
        except TypeError:
          pass
        else:
          self.fail("test_call")

    def test_getattr(self):
        """Check the use of the __getattr__ method"""
        try:
          tmp = self.instance.A.value
        except AttributeError:
          pass
        else:
          self.fail("test_call")

    def test_setattr_value(self):
        """Check the use of the __setattr__ method"""
        import coopr.pyomo.base.param
        self.instance.components.A.value = 4.3
        #print type(self.instance.components.A)
        self.instance.components.A = 4.3
        #print type(self.instance.components.A)
        if self.instance.components.A.repn_type == 'pyomo_dict':
            self.instance.A[1].value = 4.3
            self.assertEqual( type(self.instance.A[1]), coopr.pyomo.base.param._ParamValue)
        self.instance.A[1] = 4.3
        if self.instance.components.A.repn_type == 'pyomo_dict':
            self.assertEqual( type(self.instance.A[1]), coopr.pyomo.base.param._ParamValue)
        else:
            self.assertEqual( type(self.instance.A[1]), float)


    def test_setitem(self):
        """Check the use of the __setattr__ method"""
        self.instance.A[3] = 4.3
        self.assertEqual( self.instance.A[3], 4.3)
        self.instance.A[1] = 4.3
        self.assertEqual( self.instance.A[1], 4.3)
        try:
          self.instance.A[3] = 'A'
        except ValueError:
          self.fail("fail test_setitem")

    def test_keys(self):
        """Check the use of keys"""
        self.assertEqual( len(self.instance.A.keys()), 1)

    def test_dim(self):
        """Check the use of dim"""
        self.assertEqual( self.instance.components.A.dim(), 1)

    def test_len(self):
        """Check the use of len"""
        self.instance.A[3] = 4.3
        self.assertEqual( len(self.instance.A), 2)

    def test_getitem(self):
        import coopr.pyomo.base.param
        try:
            self.assertEqual( self.instance.A[1], 1.3)
        except KeyError:
           self.fail("test_getitem")

        if self.instance.components.A.repn_type == 'pyomo_dict':
            self.assertEqual(type(self.instance.A[1]), coopr.pyomo.base.param._ParamValue)
        else:
            self.assertEqual(type(self.instance.A[1]), float)

        try:
            self.instance.A[3]
        except ValueError:
            pass
        else:
          self.fail("test_getitem")


class MArrayParam1(ArrayParam1):

    def config_repn(self):
        self.repn = 'sparse_dict'

MArrayParam1 = unittest.skipIf(sys.version_info[:2] < (2,6), "Skipping tests because sparse_dict repn is not supported")(MArrayParam1)


class ArrayParam2(ArrayParam1):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        self.model.Z = Set(initialize=[1,3])
        self.model.A = Param(self.model.Z, initialize={1:1.3}, default=0.0, repn=self.repn)
        self.instance = self.model.create()

    def test_setattr_default(self):
        """Check the use of the __setattr__ method"""
        self.model.Z = Set(initialize=[1,3])
        self.model.A = Param(self.model.Z, repn=self.repn, default=4.3)
        self.instance = self.model.create()
        self.assertEqual( self.instance.A[3], 4.3)

    def test_keys(self):
        """Check the use of keys"""
        self.assertEqual( len(self.instance.A.keys()), 2)

    def test_len(self):
        """Check the use of len"""
        self.assertEqual( len(self.instance.A), 2)

    def test_getitem(self):
        try:
            self.assertEqual( self.instance.A[None], 0.0)
        except KeyError:
            pass
        else:
            if self.instance.components.A.repn_type == 'pyomo_dict':
                self.fail("test_getitem")
        self.assertEqual( self.instance.A[1], 1.3)
        self.assertEqual( self.instance.A[3], 0)


class MArrayParam2(ArrayParam2):

    def config_repn(self):
        self.repn = 'sparse_dict'

MArrayParam2 = unittest.skipIf(sys.version_info[:2] < (2,6), "Skipping tests because sparse_dict repn is not supported")(MArrayParam2)

class ArrayParam3(ArrayParam2):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        self.model.Z = Set(initialize=[1,3])
        self.model.A = Param(self.model.Z, initialize={1:1.3},default=99.0, repn=self.repn)
        self.instance = self.model.create()

    def test_len(self):
        """Check the use of len"""
        self.assertEqual( len(self.instance.A), 2)

    def test_getitem(self):
        try:
            self.assertEqual( self.instance.A[None], 0.0)
        except AssertionError:
            if self.instance.components.A.repn_type == 'pyomo_dict':
                self.fail("test_getitem")
        except KeyError:
            pass
        else:
            self.fail("test_getitem")
        self.assertEqual( self.instance.A[1], 1.3)
        self.assertEqual( self.instance.A[3], 99.0)


class MArrayParam3(ArrayParam3):

    def config_repn(self):
        self.repn = 'sparse_dict'

MArrayParam3 = unittest.skipIf(sys.version_info[:2] < (2,6), "Skipping tests because sparse_dict repn is not supported")(MArrayParam3)


class ArrayParam4(ArrayParam3):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        self.model.Z = Set(initialize=[1,3])
        self.model.A = Param(self.model.Z, initialize=1.3, repn=self.repn)
        self.instance = self.model.create()

    def test_keys(self):
        """Check the use of keys"""
        self.assertEqual( len(self.instance.A.keys()), 2)

    def test_len(self):
        """Check the use of len"""
        self.assertEqual( len(self.instance.A), 2)

    def test_getitem(self):
        try:
            self.assertEqual( self.instance.A[None], 0.0)
        except KeyError:
            pass
        else:
            self.fail("test_getitem")
        self.assertEqual( self.instance.A[1], 1.3)
        self.assertEqual( self.instance.A[3], 1.3)


class MArrayParam4(ArrayParam4):

    def config_repn(self):
        self.repn = 'sparse_dict'

MArrayParam4 = unittest.skipIf(sys.version_info[:2] < (2,6), "Skipping tests because sparse_dict repn is not supported")(MArrayParam4)


class ArrayParam5(ArrayParam4):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        self.model.Z = Set(initialize=[1,3])
        def A_init(i,model):
          return 1.3
        self.model.A = Param(self.model.Z, initialize=A_init, repn=self.repn)
        self.instance = self.model.create()


class MArrayParam5(ArrayParam5):

    def config_repn(self):
        self.repn = 'sparse_dict'

MArrayParam5 = unittest.skipIf(sys.version_info[:2] < (2,6), "Skipping tests because sparse_dict repn is not supported")(MArrayParam5)


class ArrayParam6(PyomoModel):

    def test_index1(self):
        self.model.A = Set(initialize=range(0,4))
        def B_index(model):
            for i in model.A:
                if i%2 == 0:
                    yield i
        def B_init(i,j,model):
            if j:
                return 2+i
            return -(2+i)
        self.model.B = Param(B_index, [True,False], initialize=B_init, repn=self.repn)
        self.instance = self.model.create()
        #self.instance.pprint()
        self.assertEqual(set(self.instance.B.keys()),set([(0,True),(2,True),(0,   False),(2,False)]))
        self.assertEqual(self.instance.B[0,True],2)
        self.assertEqual(self.instance.B[0,False],-2)
        self.assertEqual(self.instance.B[2,True],4)
        self.assertEqual(self.instance.B[2,False],-4)

    def test_index2(self):
        self.model.A = Set(initialize=range(0,4))
        @set_options(dimen=3)
        def B_index(model):
            return [(i,2*i,i*i) for i in model.A if i%2 == 0]
        def B_init(i,ii,iii,j,model):
            if j:
                return 2+i
            return -(2+i)
        self.model.B = Param(B_index, [True,False], initialize=B_init, repn=self.repn)
        self.instance = self.model.create()
        #self.instance.pprint()
        self.assertEqual(set(self.instance.B.keys()),set([(0,0,0,True),(2,4,4,True),(0,0,0,False),(2,4,4,False)]))
        self.assertEqual(self.instance.B[0,0,0,True],2)
        self.assertEqual(self.instance.B[0,0,0,False],-2)
        self.assertEqual(self.instance.B[2,4,4,True],4)
        self.assertEqual(self.instance.B[2,4,4,False],-4)

    def test_index3(self):
        self.model.A = Set(initialize=range(0,4))
        def B_index(model):
            return [(i,2*i,i*i) for i in model.A if i%2 == 0]
        def B_init(i,ii,iii,j,model):
            if j:
                return 2+i
            return -(2+i)
        self.model.B = Param(B_index, [True,False], initialize=B_init, repn=self.repn)
        try:
            self.instance = self.model.create()
            self.fail("Expected ValueError because B_index returns a tuple")
        except ValueError:
            pass

    def test_index4(self):
        self.model.A = Set(initialize=range(0,4))
        @set_options(within=Integers)
        def B_index(model):
            return [i/2.0 for i in model.A]
        def B_init(i,j,model):
            if j:
                return 2+i
            return -(2+i)
        self.model.B = Param(B_index, [True,False], initialize=B_init, repn=self.repn)
        try:
            self.instance = self.model.create()
            self.fail("Expected ValueError because B_index returns invalid index values")
        except ValueError:
            pass

    def test_dimen1(self):
        model=AbstractModel()
        model.A = Set(dimen=2, initialize=[(1,2),(3,4)])
        model.B = Set(dimen=3, initialize=[(1,1,1),(2,2,2),(3,3,3)])
        model.C = Set(dimen=1, initialize=[9,8,7,6,5])
        model.x = Param(model.A, model.B, model.C, initialize=-1, repn=self.repn)
        model.y = Param(model.B, initialize=(1,1), repn=self.repn)
        instance=model.create()
        self.assertEqual( instance.components.x.dim(), 6)
        self.assertEqual( instance.components.y.dim(), 3)


class MArrayParam6(ArrayParam6):

    def config_repn(self):
        self.repn = 'sparse_dict'

MArrayParam6 = unittest.skipIf(sys.version_info[:2] < (2,6), "Skipping tests because sparse_dict repn is not supported")(MArrayParam6)


class TestIO(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)

    def tearDown(self):
        if os.path.exists("param.dat"):
           os.remove("param.dat")

    def test_io1(self):
        OUTPUT=open("param.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "param A := 3.3;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.A=Param(repn=self.repn)
        self.instance = self.model.create("param.dat")
        self.assertEqual( value(self.instance.A), 3.3 )

    def test_io2(self):
        OUTPUT=open("param.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set Z := 1 3 5;"
        print >>OUTPUT, "param A :="
        print >>OUTPUT, "1 2.2"
        print >>OUTPUT, "3 2.3"
        print >>OUTPUT, "5 2.5;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.Z=Set()
        self.model.A=Param(self.model.Z, repn=self.repn)
        self.instance = self.model.create("param.dat")
        self.assertEqual( len(self.instance.A), 3 )

    def test_io3(self):
        OUTPUT=open("param.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set Z := 1 3 5;"
        print >>OUTPUT, "param : A B :="
        print >>OUTPUT, "1 2.2 3.3"
        print >>OUTPUT, "3 2.3 3.4"
        print >>OUTPUT, "5 2.5 3.5;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.Z=Set()
        self.model.A=Param(self.model.Z, repn=self.repn)
        self.model.B=Param(self.model.Z, repn=self.repn)
        self.instance = self.model.create("param.dat")
        self.assertEqual( len(self.instance.A), 3 )
        self.assertEqual( len(self.instance.B), 3 )
        self.assertEqual( self.instance.B[5], 3.5 )

    def test_io4(self):
        OUTPUT=open("param.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set Z := A1 A2 A3;"
        print >>OUTPUT, "set Y := 1 2 3;"
        print >>OUTPUT, "param A: A1 A2 A3 :="
        print >>OUTPUT, "1 1.3 2.3 3.3"
        print >>OUTPUT, "2 1.4 2.4 3.4"
        print >>OUTPUT, "3 1.5 2.5 3.5"
        print >>OUTPUT, ";"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.Z=Set()
        self.model.Y=Set()
        self.model.A=Param(self.model.Y,self.model.Z, repn=self.repn)
        self.instance = self.model.create("param.dat")
        self.assertEqual( len(self.instance.Y), 3 )
        self.assertEqual( len(self.instance.Z), 3 )
        self.assertEqual( len(self.instance.A), 9 )
        self.assertEqual( self.instance.A[1, 'A2'], 2.3 )

    def test_io5(self):
        OUTPUT=open("param.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set Z := A1 A2 A3;"
        print >>OUTPUT, "set Y := 1 2 3;"
        print >>OUTPUT, "param A (tr): A1 A2 A3 :="
        print >>OUTPUT, "1 1.3 2.3 3.3"
        print >>OUTPUT, "2 1.4 2.4 3.4"
        print >>OUTPUT, "3 1.5 2.5 3.5"
        print >>OUTPUT, ";"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.Z=Set()
        self.model.Y=Set()
        self.model.A=Param(self.model.Z,self.model.Y, repn=self.repn)
        self.instance = self.model.create("param.dat")
        self.assertEqual( len(self.instance.Y), 3 )
        self.assertEqual( len(self.instance.Z), 3 )
        self.assertEqual( len(self.instance.A), 9 )
        self.assertEqual( self.instance.A['A2',1], 2.3 )

    def test_io6(self):
        OUTPUT=open("param.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set Z := 1 3 5;"
        print >>OUTPUT, "param A default 0.0 :="
        print >>OUTPUT, "1 2.2"
        print >>OUTPUT, "3 ."
        print >>OUTPUT, "5 2.5;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.Z=Set()
        self.model.A=Param(self.model.Z, repn=self.repn)
        self.instance = self.model.create("param.dat")
        #self.instance.pprint()
        self.assertEqual( len(self.instance.components.A), 3 )
        self.assertEqual( self.instance.A[3], 0.0 )

    def test_io7(self):
        OUTPUT=open("param.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "param A := True;"
        print >>OUTPUT, "param B := False;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.A=Param(within=Boolean, repn=self.repn)
        self.model.B=Param(within=Boolean, repn=self.repn)
        self.instance = self.model.create("param.dat")
        self.assertEqual( value(self.instance.A), True )
        self.assertEqual( value(self.instance.B), False )

    def test_io8(self):
        OUTPUT=open("param.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "param : A : B :="
        print >>OUTPUT, "\"A\" 3.3"
        print >>OUTPUT, "\"B\" 3.4"
        print >>OUTPUT, "\"C\" 3.5;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.A=Set()
        self.model.B=Param(self.model.A, repn=self.repn)
        self.instance = self.model.create("param.dat")
        self.assertEqual( self.instance.A.data(), set(['A','B','C']) )

    def test_io9(self):
        OUTPUT=open("param.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "param : A : B :="
        print >>OUTPUT, "\"A\" 0.1"
        print >>OUTPUT, "\"B\" 1e-1"
        print >>OUTPUT, "\"b\" 1.4e-1"
        print >>OUTPUT, "\"C\" 1E-1"
        print >>OUTPUT, "\"c\" 1.4E-1"
        print >>OUTPUT, "\"D\" 1E+1"
        print >>OUTPUT, "\"d\" 1.4E+1"
        print >>OUTPUT, "\"AA\" -0.1"
        print >>OUTPUT, "\"BB\" -1e-1"
        print >>OUTPUT, "\"bb\" -1.4e-1"
        print >>OUTPUT, "\"CC\" -1E-1"
        print >>OUTPUT, "\"cc\" -1.4E-1"
        print >>OUTPUT, "\"DD\" -1E+1"
        print >>OUTPUT, "\"dd\" -1.4E+1;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.A=Set()
        self.model.B=Param(self.model.A, repn=self.repn)
        self.instance = self.model.create("param.dat")
        self.assertEqual( self.instance.B['A'], 0.1)
        self.assertEqual( self.instance.B['B'], 0.1)
        self.assertEqual( self.instance.B['b'], 0.14)
        self.assertEqual( self.instance.B['C'], 0.1)
        self.assertEqual( self.instance.B['c'], 0.14)
        self.assertEqual( self.instance.B['D'], 10)
        self.assertEqual( self.instance.B['d'], 14)
        self.assertEqual( self.instance.B['AA'], -0.1)
        self.assertEqual( self.instance.B['BB'], -0.1)
        self.assertEqual( self.instance.B['bb'], -0.14)
        self.assertEqual( self.instance.B['CC'], -0.1)
        self.assertEqual( self.instance.B['cc'], -0.14)
        self.assertEqual( self.instance.B['DD'], -10)
        self.assertEqual( self.instance.B['dd'], -14)

    def test_io10(self):
        OUTPUT=open("param.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set A1 := a b c d e f g h i j k l ;"
        print >>OUTPUT, "set A2 := 2 4 6 ;"
        print >>OUTPUT, "param B :="
        print >>OUTPUT, " [*,2,*] a b 1 c d 2 e f 3"
        print >>OUTPUT, " [*,4,*] g h 4 i j 5"
        print >>OUTPUT, " [*,6,*] k l 6"
        print >>OUTPUT, ";"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.A1=Set()
        self.model.A2=Set()
        self.model.B=Param(self.model.A1,self.model.A2,self.model.A1, repn=self.repn)
        self.instance = self.model.create("param.dat")
        self.assertEqual( set(self.instance.B.keys()), set([('e', 2, 'f'), ('c', 2, 'd'), ('a', 2, 'b'), ('i', 4, 'j'), ('g', 4, 'h'), ('k', 6, 'l')]))


class MTestIO(TestIO):

    def config_repn(self):
        self.repn = 'sparse_dict'

MTestIO = unittest.skipIf(sys.version_info[:2] < (2,6), "Skipping tests because sparse_dict repn is not supported")(MTestIO)


class TestParamConditional(PyomoModel):

    def test1(self):
        self.model.p = Param(initialize=1.0)
        try:
            if self.model.p:
                pass
            self.fail("Expected ValueError because parameter was undefined")
        except ValueError:
            pass
        instance = self.model.create()
        if instance.p:
            pass
        else:
            self.fail("Wrong condition value")

    def test2(self):
        self.model.p = Param(initialize=0.0)
        try:
            if self.model.p:
                pass
            self.fail("Expected ValueError because parameter was undefined")
        except ValueError:
            pass
        instance = self.model.create()
        if instance.p:
            self.fail("Wrong condition value")
        else:
            pass


class MiscParamTests(pyutilib.th.TestCase):

    def test_constructor(self):
        a = Param(name="a")
        try:
            b = Param(foo="bar")
            self.fail("Cannot pass in 'foo' as an option to Param")
        except ValueError:
            pass
        model=AbstractModel()
        model.b = Param(initialize=[1,2,3])
        try:
            model.c = Param(model.b)
            self.fail("Can't index a parameter with a parameter")
        except TypeError:
            pass
        #
        model = AbstractModel()
        model.a = Param(initialize={None:3.3})
        instance = model.create()

    def test_get_set(self):
        model=AbstractModel()
        model.a = Param()
        model.b = Set(initialize=[1,2,3])
        model.c = Param(model.b, initialize=2, within=Reals)
        #try:
            #model.a.value = 3
            #self.fail("can't set the value of an unitialized parameter")
        #except AttributeError:
            #pass
        try:
            model.a.construct()
            self.fail("Can't construct a parameter without data")
        except ValueError:
            pass
        model.a = Param(initialize=2)
        instance=model.create()
        instance.a.value=3
        #try:
            #instance.a.default='2'
            #self.fail("can't set a bad default value")
        #except ValueError:
            #pass
        self.assertEqual(2 in instance.c, True)

        try:
            instance.a[1] = 3
            self.fail("can't index a singleton parameter")
        except KeyError:
            pass
        try:
            instance.c[4] = 3
            self.fail("can't index a parameter with a bad index")
        except KeyError:
            pass
        try:
            instance.c[3] = 'a'
            self.fail("can't set a parameter with a bad value")
        except ValueError:
            pass

    def test_iter(self):
        model=AbstractModel()
        model.b = Set(initialize=[1,2,3])
        model.c = Param(model.b,initialize=2)
        instance = model.create()
        for i in instance.c:
            self.assertEqual(i in instance.c, True)

    def test_valid(self):
        def d_valid(a,model):
            return True
        def e_valid(a,i,j,model):
            return True
        model=AbstractModel()
        model.b = Set(initialize=[1,3,5])
        model.c = Param(initialize=2, within=None)
        model.d = Param(initialize=(2,3), validate=d_valid)
        model.e = Param(model.b,model.b,initialize={(1,1):(2,3)}, validate=e_valid)
        instance = model.create()
        instance.components.e.check_values()
        #try:
            #instance.c.value = 'b'
            #self.fail("can't have a non-numerical parameter")
        #except ValueError:
            #pass

if __name__ == "__main__":
   unittest.main()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________

#
# Note: this leaves the trivial constraints in the model
#

import sys
import logging
import pyutilib.component.core
from coopr.pyomo.base import Constraint, Objective
from coopr.pyomo.base import IPyomoPresolver, IPyomoPresolveAction
import coopr.pyomo.expr 

# NOTE: The idea of this module should be generaized. there are two key 
#       functionalities: computing a version of the expression tree in
#       preparation for output and identification of trivial constraints.

class ComputeCanonicalRepn(pyutilib.component.core.SingletonPlugin):
    """
    This plugin computes the canonical representation for 
    all objectives and constraints linear terms.
    """

    # all modifications are in-place, i.e., components of the
    # input instance are modified.

    pyutilib.component.core.implements(IPyomoPresolveAction)

    def __init__(self, **kwds):
        kwds['name'] = "compute_canonical_repn"
        pyutilib.component.core.Plugin.__init__(self, **kwds)

    def rank(self):
        return 10000

    def preprocess(self, model):
        """
        The main routine to perform the preprocess
        """

        if getattr(model,'skip_canonical_repn',False):
            return model
        
        #
        # Objectives
        #
        active_objectives = model.active_components(Objective)
        for key, obj in active_objectives.iteritems():
            # number of objective indicies with non-trivial expressions
            num_nontrivial = 0
            
            for ondx, objective_data in obj._data.iteritems():
                if not objective_data.active:
                    continue
                if objective_data.expr is None:
                    raise ValueError, "No expression has been defined for objective %s" % str(key)
                
                try:
                    objective_data.repn = coopr.pyomo.expr.generate_canonical_repn(objective_data.expr)
                except Exception, err:
                    logging.getLogger('coopr.pyomo').error\
                        ( "exception generating a canonical representation for objective %s (index %s): %s" \
                              % (str(key), str(ondx), str(err)) )
                    raise
                if not coopr.pyomo.expr.is_constant(objective_data.repn):
                    num_nontrivial += 1
                    
            if num_nontrivial == 0:
                obj.trivial = True
            else:
                obj.trivial = False
                
        #
        # Constraints
        #
        active_constraints = model.active_components(Constraint)
        for key, constraint in active_constraints.iteritems():

            # number of constraint indicies with non-trivial bodies
            num_nontrivial = 0
            
            for index, constraint_data in constraint._data.iteritems():

                if not constraint_data.active:
                    continue

                if constraint_data.lin_body is not None:
                   # if we already have the linear encoding of the constraint body, skip canonical expression 
                   # generation but we still need to assess constraint triviality.
                   if not coopr.pyomo.expr.is_linear_expression_constant(constraint_data.lin_body):
                      num_nontrivial += 1          
                   continue

                if constraint_data.body is None:
                    raise ValueError, "No expression has been defined for the body of constraint %s, index=%s" % (str(key), str(index))
                
                try:
                    constraint_data.repn = coopr.pyomo.expr.generate_canonical_repn(constraint_data.body)
                except Exception:
                    logging.getLogger('coopr.pyomo').error\
                        ( "exception generating a canonical representation for constraint %s (index %s)" \
                              % (str(key), str(index)) )
                    raise
                if not coopr.pyomo.expr.is_constant(constraint_data.repn):
                    num_nontrivial += 1
                    
            if num_nontrivial == 0:
                constraint.trivial = True
            else:
                constraint.trivial = False
        #
        # Return modified instance
        #
        return model




import sys
import logging
from pyutilib.misc import LogHandler

from os.path import abspath, dirname, join, normpath
coopr_base = normpath(join(dirname(abspath(__file__)), '..', '..', '..'))

logger = logging.getLogger('coopr.pyomo')
logger.setLevel( logging.WARNING )
logger.addHandler( LogHandler(coopr_base, verbosity=lambda: logger.isEnabledFor(logging.DEBUG)) )





__all__ = ['_ParamValue']

from numvalue import NumericConstant
from pyutilib.component.core import *
from plugin import IParamRepresentation
from pyutilib.misc import SparseMapping
from numvalue import create_name


class SparseMappingRepn(Plugin):

    implements(IParamRepresentation)

    alias('sparse_dict', 'A class that manages the sparse dictionary parameter representation, which maps keys to numeric values while respecting default values')

    def __init__(self, param):
        self._paramdata = SparseMapping(default=param._default.value, index=param._index, within=param.domain)
        self._param = param

    def model_repn(self):
        if None in self.keys():
            return self._paramdata[None]
        return self._paramdata

    def set_default(self, value):
        self._paramdata.default = value

    def update_index(self):
        self._paramdata.index = self._param._index

    def data(self):
        return self._paramdata

    def keys(self, nondefault=False):
        if nondefault:
            self._paramdata.nondefault_keys()
        return self._paramdata.keys()

    def __contains__(self, key):
        return key in self._paramdata

    def __iter__(self):
        return self._paramdata.__iter__()

    def __len__(self):
        return len(self._paramdata)

    def __getitem__(self, key):
        return self._paramdata[key]

    def __setitem__(self,key,val):
        self._paramdata[key] = val

    def set_item(self, key=None, val=None):
        if key is None:
            self._param._default.value = val
            self._paramdata = SparseMapping(default=self._param._default.value, index=self._param._index, within=self._param.domain)
        else:
            self._paramdata.set_item(key,val)

    def __call__(self, key):
        return self._paramdata[key]


class _ParamValue(NumericConstant):
    """Holds the numeric value of a parameter"""

    __slots__ = []

    """Constructor"""
    def __init__(self, name=None, domain=None, value=None):
        try:
            NumericConstant.__init__(self, name, domain, value)
        except ValueError, e:
            msg = "Problem constructing parameter %s : %s"
            raise ValueError, msg % (name, str(e))

    def fixed_value(self):
        return True

    def __str__(self):
        return self.name


class DefaultRepn(Plugin):

    implements(IParamRepresentation)

    alias('pyomo_dict', 'A class that manages the default Pyomo parameter representation, which maps keys to ParamValue objects')

    def __init__(self, param):
        self._paramdata = {}
        self._default = param._default
        self._index = param._index
        self._param = param

    def model_repn(self):
        return self._param

    def set_default(self, value):
        pass

    def update_index(self):
        self._index = self._param._index

    def data(self):
        tmp = SparseMapping(default=self._default.value, index=self._param._index, within=self._param.domain)
        for key in self._paramdata.keys():
            tmp[key] = self._paramdata[key].value
        return tmp

    def keys(self, nondefault=False):
        if nondefault or self._default.value is None:
            return self._paramdata.keys() 
        return self._index

    def __contains__(self, key):
        if self._default.value is None:
            return key in self._paramdata
        else:
            return key in self._index

    def __iter__(self):
        if self._default.value is None:
            return self._paramdata.keys().__iter__()
        else:
            return self._index.__iter__()

    def __len__(self):
        if self._default.value is None:
            return len(self._paramdata)
        return len(self._index)

    def __getitem__(self, key):
        if key in self._paramdata:
            return self._paramdata[key]
        if key in self._index and self._default.value is not None:
            return self._default
        if key in self._index:
            raise ValueError, "Legal key '%s' for parameter '%s', but value is uninitialized and there is no default value" % (key, self._param.name)
        raise KeyError, "Undefined key '%s' for parameter '%s'" % (key, self._param.name)

    def __setitem__(self,key,val):
        if key in self._paramdata:
            self._paramdata[key].value = val
        else:
            self._paramdata[key]= _ParamValue(
                        name   = create_name(self._param.name,key),
                        domain = self._param.domain,
                        value  = val
                        )

    def set_item(self, key=None, val=None):
        if key is None:
            self._paramdata = {}
            self._default.value = getattr(val,'value',val)
        else:
            self._paramdata[key] = val

    def __call__(self, key):
        return self.__getitem__(key).value





#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Model', 'ConcreteModel', 'AbstractModel']

import array
import copy
import logging
import re
import sys
import traceback

from plugin import *
from numvalue import *

import pyutilib.component.core
from pyutilib.math import *
from pyutilib.misc import quote_split, tuplize, Container, PauseGC

from coopr.opt import ProblemFormat, ResultsFormat, guess_format
from coopr.opt.results import SolutionMap
from coopr.pyomo.base.var import _VarValue, VarStatus, Var
from coopr.pyomo.base.constraint import ConstraintData
from coopr.pyomo.base.set_types import *
import coopr.opt
from PyomoModelData import ModelData

from block import Block
from sets import Set

try:
    from pympler.muppy import tracker
    pympler_available = True
except ImportError:
    pympler_available = False

logger = logging.getLogger('coopr.pyomo')

class Model(Block):
    """
    An optimization model.  By default, this defers construction of components
    until data is loaded.
    """

    pyutilib.component.core.alias("Model", 'Model objects can be used as a '  \
                                  'component of other models.')

    preprocessor_ep = pyutilib.component.core.ExtensionPoint(IPyomoPresolver)

    def __init__ ( self, name='unknown', _deprecate=True, **kwargs ):
        """Constructor"""
        if _deprecate:
            msg = "Using the 'Model' class is deprecated.  Please use the "    \
                  "'AbstractModel' class instead."
            logger.warning( msg )

        Block.__init__(self, ctype=Model)

        # cached name-based object lookup data to facilitate fast solution loading.
        # handled by the various objects (the items in the map) during construction.
        self._name_varmap={} # maps labels to _VarValue objects
        self._name_conmap={} # maps labels to ConstraintData objects
        self._name_objmap={} # maps labels to ObjectiveData objects

        self.name=name
        self.tmpctr=0
        self._varctr=0
        self._preprocessors = ["simple_preprocessor"]
        #
        # Dictionaries that map from id to Variable/Constraints
        #
        self._var = {}
        self._con = {}
        self._obj = {}
        #
        # Model statistics
        #
        self.statistics = Container()
        #
        # We make the default behavior of has_capabilities to lie and return
        # True. We do this to allow the richest output formats when no
        # solver is specified. These capabilities _will_ be overwritten
        # when a solver is specified.
        #
        # We define _tempTrue since using self.has_capability = lambda x: True
        # causes pickling errors.
        #
        self.has_capability = _tempTrue

    def nvariables(self):
        return self.statistics.number_of_variables

    def variable(self, name):
        if isinstance(name,basestring):
            return self._name_varmap[name]
        else:
            return self._var[name-1]

    def has_discrete_variables(self):

        variables = self.active_components(Var)
        for variable_name, variable in variables.items():
            if isinstance(variable.domain, IntegerSet) or isinstance(variable.domain, BooleanSet):
                return True
        return False


    def variables(self):
        return self._name_varmap

    def nconstraints(self):
        return self.statistics.number_of_constraints

    def constraint(self, name):
        if isinstance(name,basestring):
            return self._name_conmap[name]
        else:
            return self._con[name-1]

    def constraints(self):
        return self._name_conmap

    def nobjectives(self):
        return self.statistics.number_of_objectives

    def objective(self, name):
        if isinstance(name,basestring):
            return self._name_objmap[name]
        else:
            return self._obj[name-1]

    def objectives(self):
        return self._name_objmap

    def num_used_variables(self):
        return len(self._var)

    def valid_problem_types(self):
        return [ProblemFormat.pyomo]

    def create(self, filename=None, name=None, namespace=None, namespaces=None, preprocess=True, simplify=None, profile_memory=0):
        """
        Create a concrete instance of this Model, possibly using data
        read in from a file.
        """

        if simplify is not None:
            print """
 WARNING: The 'simplify' argument to PyomoModel.create() has been deprecated.
      Please remove references from your code.
 """
        if self._defer_construction:
            instance = self.clone()

            if namespaces is None:
                instance.load(filename, namespaces=[namespace], profile_memory=profile_memory)
            else:
                instance.load(filename, namespaces=namespaces, profile_memory=profile_memory)
        else:
            instance = self
        if preprocess is True:

            if (pympler_available is True) and (profile_memory >= 2):
                memory_tracker = tracker.SummaryTracker()
                memory_tracker.create_summary()

            instance.preprocess()

            if (pympler_available is True) and (profile_memory >= 2):
                print "Objects created during instance preprocessing:"
                memory_tracker.print_diff()

        if not name is None:
            instance.name=name
        return instance

    def clone(self):
        """Create a copy of this model"""

        for name in self.components:
            #print 'y',name
            #print 'z', type(self.components[name])
            self.components[name].model = None

        instance = copy.deepcopy(self)

        for name in self.components:
            self.components[name].model = self
        for name in instance.components:
            instance.components[name].model = instance
        return instance

    def reset(self):
        for name in self.components:
            self.components[name].reset()

    def preprocess(self):
        """Apply the preprocess plugins defined by the user"""
        suspend_gc = PauseGC()
        for item in self._preprocessors:
            self = Model.preprocessor_ep.service(item).preprocess(self)

    def solution(selfNone):
        """Return the solution"""
        soln = []
        for i in range(len(self._var)):
            soln.append( self._var[i].value )
        return soln

    def update_results(self, results):
        for i in xrange(len(results.solution)):
            soln = results.solution(i+1)
            #
            # Variables
            #
            vars = SolutionMap()
            flag=False
            for name in soln.variable.keys():
                flag=True
                entry = soln.variable[name]
                # NOTE: the following is a hack, to handle the ONE_VAR_CONSTANT variable
                if name == "ONE_VAR_CONSTANT":
                    continue
                # translate the name first if there is a variable map associated
                # with the input solution.
                if results.symbol_map is not None:
                    name = results.symbol_map.get(name, name)
                if name not in self._name_varmap:
                    msg = "Variable name '%s' is not in model '%s'.  Valid "      \
                        'variable names: %s'
                    raise KeyError, msg % (
                          name, self.name, str(self._name_varmap.keys()) )
                var_value = self._name_varmap[name]
                entry.id = var_value.id
                vars.declare(name)
                dict.__setitem__(vars,name, entry)
            if flag:
                dict.__setitem__(soln, 'Variable', vars)
            #
            # Constraints
            #
            cons = SolutionMap()
            flag=False
            for name in soln.constraint.keys():
                flag=True
                entry = soln.constraint[name]
                if results.symbol_map is not None:
                    name = results.symbol_map.get(name, name)
                if name not in self._name_conmap:
                    if name.endswith('ONE_VAR_CONSTANT'):
                        # Should this data be filtered from the results object?
                        continue
                    msg = "Constraint name '%s' is not in model '%s'.  Valid "      \
                        'constraint names: %s'
                    raise KeyError, msg % (
                          name, self.name, str(self._name_conmap.keys()) )
                con_value = self._name_conmap[name]
                entry.id = con_value.id
                cons.declare(name)
                dict.__setitem__(cons,name, entry)
            if flag:
                dict.__setitem__(soln, 'Constraint', cons)
            #
            # Objectives
            #
            objs = SolutionMap()
            flag=False
            for name in soln.objective.keys():
                flag=True
                entry = soln.objective[name]
                if results.symbol_map is not None:
                    name = results.symbol_map.get(name, name)
                if not name in self._name_objmap:
                    msg = "Objective name '%s' is not in model '%s'.  Valid "      \
                        'objective names: %s'
                    raise KeyError, msg % (
                          name, self.name, str(self._name_objmap.keys()) )
                obj_value = self._name_objmap[name]
                entry.id = obj_value.id
                objs.declare(name)
                dict.__setitem__(objs,name, entry)
            if flag:
                dict.__setitem__(soln, 'Objective', objs)

    def load(self, arg, namespaces=[None], simplify=None, profile_memory=0):
        """ Load the model with data from a file or a Solution object """

        if simplify is not None:
            print """
 WARNING: The 'simplify' argument to PyomoModel.load() has been deprecated.
      Please remove references from your code.
 """

        if arg is None or type(arg) is str:
            self._load_model_data(ModelData(filename=arg,model=self), namespaces, profile_memory=profile_memory)
            return True
        elif type(arg) is ModelData:
            self._load_model_data(arg, namespaces, profile_memory=profile_memory)
            return True
        elif type(arg) is coopr.opt.SolverResults:
            # if the solver status not one of either OK or Warning, then error.
            if (arg.solver.status != coopr.opt.SolverStatus.ok) and \
               (arg.solver.status != coopr.opt.SolverStatus.warning):
                msg = 'Cannot load a SolverResults object with bad status: %s'
                raise ValueError, msg % str( arg.solver.status )

            # but if there is a warning, print out a warning, as someone should
            # probably take a look!
            if (arg.solver.status == coopr.opt.SolverStatus.warning):
                print 'WARNING - Loading a SolverResults object with a '       \
                      'warning status'

            if len(arg.solution) > 0:
                self._load_solution(arg.solution(0),symbol_map=arg.symbol_map)
                return True
            else:
                return False
        elif type(arg) is coopr.opt.Solution:
            self._load_solution(arg)
            return True
        elif type(arg) in (tuple, list):
            self._load_solution(arg)
            return True
        else:
            msg = "Cannot load model with object of type '%s'"
            raise ValueError, msg % str( type(arg) )

    def store_info(self, results):
        """ Store model information into a SolverResults object """
        results.problem.name = self.name

        stat_keys = (
          'number_of_variables',
          'number_of_binary_variables',
          'number_of_integer_variables',
          'number_of_continuous_variables',
          'number_of_constraints',
          'number_of_objectives'
        )

        for key in stat_keys:
            results.problem.__dict__[key] = self.statistics.__dict__[key]

    def _tuplize(self, data, setobj):
        if data is None:            #pragma:nocover
            return None
        if setobj.dimen == 1:
            return data
        ans = {}
        for key in data:
            if type(data[key][0]) is tuple:
                return data
            ans[key] = tuplize(data[key], setobj.dimen, setobj.name)
        return ans

    def _load_model_data(self, modeldata, namespaces, **kwds):
        """
        Load declarations from a ModelData object.
        """

        # As we are primarily generating objects here (and acyclic ones
        # at that), there is no need to run the GC until the entire
        # model is created.  Simple reference-counting should be
        # sufficient to keep memory use under control.
        suspend_gc = PauseGC()

        profile_memory = kwds.get('profile_memory', 0)

        # for pretty-printing summaries - unlike the standard
        # method in the pympler summary module, the tracker
        # doesn't print 0-byte entries to pad out the limit.
        if (pympler_available is True) and (profile_memory >= 2):
            memory_tracker = tracker.SummaryTracker()

        # Do some error checking
        for namespace in namespaces:
            if not namespace is None and not namespace in modeldata._data:
                msg = "Cannot access undefined namespace: '%s'"
                raise IOError, msg % namespace

        # Initialize each component in order.
        components = [key for key in self.components]

        for component_name in components:

            if (pympler_available is True) and (profile_memory >= 2):
                memory_tracker.create_summary()

            if self.components[component_name].type() is Model:
                continue
            declaration = self.components[component_name]
            if component_name in modeldata._default.keys():
                if declaration.type() is Set:
                    declaration.set_default(self._tuplize(modeldata._default[component_name], declaration))
                else:
                    declaration.set_default(modeldata._default[component_name])
            data = None

            for namespace in namespaces:
                if component_name in modeldata._data.get(namespace,{}).keys():
                    if declaration.type() is Set:
                        data = self._tuplize(modeldata._data[namespace][component_name],
                                             declaration)
                    else:
                        data = modeldata._data[namespace][component_name]
                if not data is None:
                    break

            if __debug__:
                if logger.isEnabledFor(logging.DEBUG):
                    msg = "About to generate '%s' with data: %s"
                    print msg % ( declaration.name, str(data) )
                    self.pprint()

            try:
                declaration.construct(data)
            except:
                logger.error("constructing declaration '%s' from data=%s failed",
                             str(declaration.name), str(data).strip() )
                raise

            if (pympler_available is True) and (profile_memory >= 2):
                print "Objects created during construction of component "+component_name+":"
                memory_tracker.print_diff()

    def _load_solution(self, soln, symbol_map=None):
        """
        Load a solution.
        """
        # Load list or tuple into the variables, based on their order
        if type(soln) in (list, tuple):
            if len(soln) != len(self._var):
                msg = 'Attempting to load a list/tuple solution into a '      \
                      'model, but its length is %d while the model has %d '   \
                      'variables' % (len(soln), len(self._var))
                raise ValueError, msg % ( len(soln), len(self._var) )
            for i in range(len(soln)):
                self._var[i].value = soln[i]
            return
        #
        # Load variable data
        #
        for name, entry in soln.variable.iteritems():

            # NOTE: the following is a hack, to handle the ONE_VAR_CONSTANT
            #    variable that is necessary for the objective constant-offset
            #    terms.  Probably should create a dummy variable in the model
            #    map at the same time the objective expression is being
            #    constructed.

            if name != "ONE_VAR_CONSTANT":

                # translate the name first if there is a variable map associated
                # with the input solution.
                if symbol_map is not None:
                    name = symbol_map.get(name, name)
                if name not in self._name_varmap:
                    msg = "Variable name '%s' is not in model '%s'.  Valid "      \
                          'variable names: %s'
                    raise KeyError, msg % (
                              name, self.name, str(self._name_varmap.keys()) )

                if not isinstance(self._name_varmap[name],_VarValue):
                    msg = "Variable '%s' in model '%s' is type %s"
                    raise TypeError, msg % (
                        name, self.name, str(type(self._name_varmap[name])) )

                if self._name_varmap[name].fixed is True:
                    msg = "Variable '%s' in model '%s' is currently fixed - new" \
                          ' value is not expected in solution'
                    raise TypeError, msg % ( name, self.name )

                if self._name_varmap[name].status is not VarStatus.used:
                    msg = "Variable '%s' in model '%s' is not currently used - no" \
                          ' value is expected in solution'
                    raise TypeError, msg % ( name, self.name )

                var_value = self._name_varmap[name]
                for _key in entry.keys():
                    key = _key[0].lower() + _key[1:]
                    if key == 'value':
                        var_value.value = entry.value
                    elif not key == 'id':
                        var_value.setattrvalue(key, getattr(entry, _key))
        #
        # Load constraint data
        #
        for name,entry in soln.constraint.iteritems():
                #
                # This is a hack - see above.
                #
            if name.endswith('ONE_VAR_CONSTANT'):
                continue
            if symbol_map is not None:
                name = symbol_map.get(name, name)
            #
            # This is a hack.  Is there a standard convention for constraint
            # names that we can apply to all data formats?
            #
            if name[0:2] == 'c_':
                _name = name[4:-1]
            else:
                _name = name

            if _name not in self._name_conmap:
                msg = "Constraint name '%s' is not in model '%s'.  Valid "    \
                      'constraint names: %s'
                raise KeyError, msg % (
                              _name, self.name, str(self._name_conmap.keys()) )

            if not isinstance(self._name_conmap[_name],ConstraintData):
                msg = "Constraint '%s' in model '%s' is type %s"
                raise TypeError, msg % (
                       name, self.name, str(type(self._name_conmap[_name])) )

            constraint_data = self._name_conmap[_name]
            for _key in entry.keys():
                key = _key[0].lower() + _key[1:]
                if key == 'value':
                    constraint_data.value = entry.value
                elif not key == 'id':
                    setattr(constraint_data, key, getattr(entry, _key))

    def write(self,filename=None,format=ProblemFormat.cpxlp):
        """
        Write the model to a file, with a given format.
        """
        if format is None and not filename is None:
            #
            # Guess the format
            #
            format = guess_format(filename)
        #print "X",str(format),coopr.opt.WriterFactory.services()
        problem_writer = coopr.opt.WriterFactory(format)
        if problem_writer is None:
            msg = "Cannot write model in format '%s': no model writer "      \
                  'registered for that format'
            raise ValueError, msg % str(format)

        (fname, symbol_map) = problem_writer(self, filename)
        if __debug__:
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Writing model '%s' to file '%s' with format %s", self.name, str(fname), str(format))
        return fname, symbol_map

    def to_standard_form(self):
        """
        Produces a standard-form representation of the model. Returns
        the coefficient matrix (A), the cost vector (c), and the
        constraint vector (b), where the 'standard form' problem is

        min/max c'x
        s.t.    Ax = b
                x >= 0

        All three returned values are instances of the array.array
        class, and store Python floats (C doubles).
        """

        from coopr.pyomo.expr import generate_canonical_repn


        # We first need to create an map of all variables to their column
        # number
        colID = {}
        ID2name = {}
        id = 0
        tmp = self.variables().keys()
        tmp.sort()

        for v in tmp:
            colID[v] = id
            ID2name[id] = v
            id += 1

        # First we go through the constraints and introduce slack and excess
        # variables to eliminate inequality constraints
        #
        # N.B. Structure heirarchy:
        #
        # active_components: {class: {attr_name: object}}
        # object -> Constraint: ._data: {ndx: ConstraintData}
        # ConstraintData: .lower, .body, .upper
        #
        # So, altogether, we access a lower bound via
        #
        # model.active_components()[Constraint]['con_name']['index'].lower
        #
        # {le,ge,eq}Constraints are
        # {constraint_name: {index: {variable_or_none: coefficient}} objects
        # that represent each constraint. None in the innermost dictionary
        # represents the constant term.
        #
        # i.e.
        #
        # min  x1 + 2*x2 +          x4
        # s.t. x1                         = 1
        #           x2   + 3*x3          <= -1
        #      x1 +                 x4   >= 3
        #      x1 + 2*x2 +      +   3*x4 >= 0
        #
        #
        # would be represented as (modulo the names of the variables,
        # constraints, and indices)
        #
        # eqConstraints = {'c1': {None: {'x1':1, None:-1}}}
        # leConstraints = {'c2': {None: {'x2':1, 'x3':3, None:1}}}
        # geConstraints = {'c3': {None: {'x1':1, 'x4':1, None:-3}},
        #                  'c4': {None: {'x1':1, 'x2':2, 'x4':1, None:0}}}
        #
        # Note the we have the luxury of dealing only with linear terms.
        leConstraints = {}
        geConstraints = {}
        eqConstraints = {}
        objectives = {}
        # For each registered component
        for c in self.active_components():

            # Get all subclasses of ConstraintBase
            if issubclass(c, ConstraintBase):
                cons = self.active_components(c)

                # Get the name of the constraint, and the constraint set itself
                for con_set_name in cons:
                    con_set = cons[con_set_name]

                    # For each indexed constraint in the constraint set
                    for ndx in con_set._data:
                        con = con_set._data[ndx]

                        # Process the body
                        terms = self._process_canonical_repn(
                            generate_canonical_repn(con.body))

                        # Process the bounds of the constraint
                        if con._equality:
                            # Equality constraint, only check lower bound
                            lb = self._process_canonical_repn(
                                generate_canonical_repn(con.lower))

                            # Update terms
                            for k in lb:
                                v = lb[k]
                                if k in terms:
                                    terms[k] -= v
                                else:
                                    terms[k] = -v

                            # Add constraint to equality constraints
                            eqConstraints[(con_set_name, ndx)] = terms
                        else:

                            # Process upper bounds (<= constraints)
                            if con.upper is not None:
                                # Less than or equal to constraint
                                tmp = dict(terms)

                                ub = self._process_canonical_repn(
                                    generate_canonical_repn(con.upper))

                                # Update terms
                                for k in ub:
                                    if k in terms:
                                        tmp[k] -= ub[k]
                                    else:
                                        tmp[k] = -ub[k]

                                # Add constraint to less than or equal to
                                # constraints
                                leConstraints[(con_set_name, ndx)] = tmp

                            # Process lower bounds (>= constraints)
                            if con.lower is not None:
                                # Less than or equal to constraint
                                tmp = dict(terms)

                                lb = self._process_canonical_repn(
                                    generate_canonical_repn(con.lower))

                                # Update terms
                                for k in lb:
                                    if k in terms:
                                        tmp[k] -= lb[k]
                                    else:
                                        tmp[k] = -lb[k]

                                # Add constraint to less than or equal to
                                # constraints
                                geConstraints[(con_set_name, ndx)] = tmp
            elif issubclass(c, Objective):
                # Process objectives
                objs = self.active_components(c)

                # Get the name of the objective, and the objective set itself
                for obj_set_name in objs:
                    obj_set = objs[obj_set_name]

                    # For each indexed objective in the objective set
                    for ndx in obj_set._data:
                        obj = obj_set._data[ndx]
                        # Process the objective
                        terms = self._process_canonical_repn(
                            generate_canonical_repn(obj.expr))

                        objectives[(obj_set_name, ndx)] = terms


        # We now have all the constraints. Add a slack variable for every
        # <= constraint and an excess variable for every >= constraint.
        nSlack = len(leConstraints)
        nExcess = len(geConstraints)

        nConstraints = len(leConstraints) + len(geConstraints) + \
                       len(eqConstraints)
        nVariables = len(colID) + nSlack + nExcess
        nRegVariables = len(colID)

        # Make the arrays
        coefficients = array.array("d", [0]*nConstraints*nVariables)
        constraints = array.array("d", [0]*nConstraints)
        costs = array.array("d", [0]*nVariables)

        # Populate the coefficient matrix
        constraintID = 0

        # Add less than or equal to constraints
        for ndx in leConstraints:
            con = leConstraints[ndx]
            for termKey in con:
                coef = con[termKey]

                if termKey is None:
                    # Constraint coefficient
                    constraints[constraintID] = -coef
                else:
                    # Variable coefficient
                    col = colID[termKey]
                    coefficients[constraintID*nVariables + col] = coef

            # Add the slack
            coefficients[constraintID*nVariables + nRegVariables + \
                        constraintID] = 1
            constraintID += 1

        # Add greater than or equal to constraints
        for ndx in geConstraints:
            con = geConstraints[ndx]
            for termKey in con:
                coef = con[termKey]

                if termKey is None:
                    # Constraint coefficient
                    constraints[constraintID] = -coef
                else:
                    # Variable coefficient
                    col = colID[termKey]
                    coefficients[constraintID*nVariables + col] = coef

            # Add the slack
            coefficients[constraintID*nVariables + nRegVariables + \
                        constraintID] = -1
            constraintID += 1

        # Add equality constraints
        for ndx in eqConstraints:
            con = eqConstraints[ndx]
            for termKey in con:
                coef = con[termKey]

                if termKey is None:
                    # Constraint coefficient
                    constraints[constraintID] = -coef
                else:
                    # Variable coefficient
                    col = colID[termKey]
                    coefficients[constraintID*nVariables + col] = coef

            constraintID += 1

        # Determine cost coefficients
        for obj_name in objectives:
            obj = objectives[obj_name]
            for var in obj:
                costs[colID[var]] = obj[var]

        # Print the model
        #
        # The goal is to print
        #
        #         var1   var2   var3   ...
        #       +--                     --+
        #       | cost1  cost2  cost3  ...|
        #       +--                     --+
        #       +--                     --+ +-- --+
        # con1  | coef11 coef12 coef13 ...| | eq1 |
        # con2  | coef21 coef22 coef23 ...| | eq2 |
        # con2  | coef31 coef32 coef33 ...| | eq3 |
        #  .    |   .      .      .   .   | |  .  |
        #  .    |   .      .      .    .  | |  .  |
        #  .    |   .      .      .     . | |  .  |

        constraintPadding = 2
        numFmt = "% 1.4f"
        altFmt = "% 1.1g"
        maxColWidth = max(len(numFmt % 0.0), len(altFmt % 0.0))
        maxConstraintColWidth = max(len(numFmt % 0.0), len(altFmt % 0.0))

        # Generate constraint names
        maxConNameLen = 0
        conNames = []
        for name in leConstraints:
            strName = str(name)
            if len(strName) > maxConNameLen:
                maxConNameLen = len(strName)
            conNames.append(strName)
        for name in geConstraints:
            strName = str(name)
            if len(strName) > maxConNameLen:
                maxConNameLen = len(strName)
            conNames.append(strName)
        for name in eqConstraints:
            strName = str(name)
            if len(strName) > maxConNameLen:
                maxConNameLen = len(strName)
            conNames.append(strName)

        # Generate the variable names
        varNames = [None]*len(colID)
        for name in colID:
            tmp_name = " " + name
            if len(tmp_name) > maxColWidth:
                maxColWidth = len(tmp_name)
            varNames[colID[name]] = tmp_name
        for i in xrange(0, nSlack):
            tmp_name = " _slack_%i" % i
            if len(tmp_name) > maxColWidth:
                maxColWidth = len(tmp_name)
            varNames.append(tmp_name)
        for i in xrange(0, nExcess):
            tmp_name = " _excess_%i" % i
            if len(tmp_name) > maxColWidth:
                maxColWidth = len(tmp_name)
            varNames.append(tmp_name)

        # Variable names
        line = " "*maxConNameLen + (" "*constraintPadding) + " "
        for col in xrange(0, nVariables):
            # Format entry
            token = varNames[col]

            # Pad with trailing whitespace
            token += " "*(maxColWidth - len(token))

            # Add to line
            line += " " + token + " "
        print line

        # Cost vector
        print " "*maxConNameLen + (" "*constraintPadding) + "+--" + \
              " "*((maxColWidth+2)*nVariables - 4) + "--+"
        line = " "*maxConNameLen + (" "*constraintPadding) + "|"
        for col in xrange(0, nVariables):
            # Format entry
            token = numFmt % costs[col]
            if len(token) > maxColWidth:
                token = altFmt % costs[col]

            # Pad with trailing whitespace
            token += " "*(maxColWidth - len(token))

            # Add to line
            line += " " + token + " "
        line += "|"
        print line
        print " "*maxConNameLen + (" "*constraintPadding) + "+--" + \
              " "*((maxColWidth+2)*nVariables - 4) + "--+"

        # Constraints
        print " "*maxConNameLen + (" "*constraintPadding) + "+--" + \
              " "*((maxColWidth+2)*nVariables - 4) + "--+" + \
              (" "*constraintPadding) + "+--" + \
              (" "*(maxConstraintColWidth-1)) + "--+"
        for row in xrange(0, nConstraints):
            # Print constraint name
            line = conNames[row] + (" "*constraintPadding) + (" "*(maxConNameLen - len(conNames[row]))) + "|"

            # Print each coefficient
            for col in xrange(0, nVariables):
                # Format entry
                token = numFmt % coefficients[nVariables*row + col]
                if len(token) > maxColWidth:
                    token = altFmt % coefficients[nVariables*row + col]

                # Pad with trailing whitespace
                token += " "*(maxColWidth - len(token))

                # Add to line
                line += " " + token + " "

            line += "|" + (" "*constraintPadding) + "|"

            # Add constraint vector
            token = numFmt % constraints[row]
            if len(token) > maxConstraintColWidth:
                token = altFmt % constraints[row]

            # Pad with trailing whitespace
            token += " "*(maxConstraintColWidth - len(token))

            line += " " + token + "  |"
            print line
        print " "*maxConNameLen + (" "*constraintPadding) + "+--" + \
              " "*((maxColWidth+2)*nVariables - 4) + "--+" + \
              (" "*constraintPadding) + "+--" + (" "*(maxConstraintColWidth-1))\
              + "--+"

        return (coefficients, costs, constraints)

    def _process_canonical_repn(self, expr):
        """
        Returns a dictionary of {var_name_or_None: coef} values
        """

        terms = {}

        # Get the variables from the canonical representation
        vars = expr.pop(-1, {})

        # Find the linear terms
        linear = expr.pop(1, {})
        for k in linear:
            # FrozeDicts don't support (k, v)-style iteration
            v = linear[k]

            # There's exactly 1 variable in each term
            terms[vars[k.keys()[0]].label] = v

        # Get the constant term, if present
        const = expr.pop(0, {})
        if None in const:
            terms[None] = const[None]

        if len(expr) != 0:
            raise TypeError, "Nonlinear terms in expression"

        return terms


def _tempTrue(x):
    """
    Defined to return True to all inquiries
    """
    return True


class ConcreteModel(Model):
    """
    A concrete optimization model that does not defer construction of
    components.
    """

    def __init__(self, *args, **kwds):
        kwds['_deprecate'] = False
        Model.__init__(self, *args, **kwds)
        self._defer_construction=False

    pyutilib.component.core.alias("ConcreteModel", 'A concrete optimization model that '\
              'does not defer construction of components.')

class AbstractModel(Model):
    """
    An abstract optimization model that defers construction of
    components.
    """

    def __init__(self, *args, **kwds):
        kwds['_deprecate'] = False
        Model.__init__(self, *args, **kwds)
        self._defer_construction=True

    pyutilib.component.core.alias("AbstractModel", 'An abstract optimization model that '\
              'defers construction of components.')




__all__ = ['Component']

from pyutilib.component.core import Plugin, implements
from plugin import IModelComponent


class Component(Plugin):

    implements(IModelComponent)

    def __init__ ( self, ctype=None, **kwargs ):
        # Error check for ctype
        if ctype is None:
            raise DeveloperError, "Must specify a class for the component type!"

        self.active=True
        self._type=ctype
        self._constructed=False
        self.model=None

    def activate(self):
        self.active=True

    def deactivate(self):
        self.active=False

    def type(self):
        return self._type

    def construct(self, data=None):
        pass

    def is_constructed(self):
       return self._constructed


class DeveloperError(Exception):
    """
    Exception class used to throw errors stemming from Pyomo
    programming errors, rather than user modeling errors (e.g., a
    component not declaring a 'ctype')

    """

    def __init__(self, val):
        self.parameter = val

    def __str__(self):
        return repr(self.parameter)



#
# Test the canonical expressions 
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+os.sep+".."+os.sep+"..")
currdir = dirname(abspath(__file__))+os.sep

from coopr.pyomo import *
from coopr.opt import *
#from coopr.pyomo.base.var import _VarElement
import pyutilib.th as unittest
import pyutilib.services
from coopr.pyomo.preprocess.simple_preprocessor import SimplePreprocessor
from pyutilib.component.core import PluginGlobals

class frozendict(dict):
    __slots__ = ('_hash',)
    def __hash__(self):
        rval = getattr(self, '_hash', None)
        if rval is None:
            rval = self._hash = hash(frozenset(self.iteritems()))
        return rval


class TestBase(unittest.TestCase):

    def setUp(self):
        self.model = AbstractModel()

    def construct(self,filename):
        self.instance = self.model.create(filename)


class Test(TestBase):

    def setUp(self):
        #
        # Create Model
        #
        TestBase.setUp(self)
        #PluginGlobals.pprint()
        self.plugin = SimplePreprocessor()
        self.plugin.deactivate_action("compute_canonical_repn")

    def tearDown(self):
        if os.path.exists("unknown.lp"):
           os.unlink("unknown.lp")
        pyutilib.services.TempfileManager.clear_tempfiles()
        self.plugin.activate_action("compute_canonical_repn")

    def test_expr1(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return summation(model.p, model.x)
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        #self.instance.obj[None].expr.pprint()
        mid=id(self.instance)
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        del rep[-1]
        self.assertEqual(rep, {1: {frozendict({(mid,2): 1}): 6.0, frozendict({(mid,1): 1}): 4.0, frozendict({(mid,3): 1}): 8.0, frozendict({(mid,0): 1}): 2.0}})

    def test_expr2(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return summation(model.p, model.x)
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        mid=id(self.instance)
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        del rep[-1]
        self.assertEqual(rep, {1: {frozendict({(mid,2): 1}): 6.0, frozendict({(mid,1): 1}): 4.0, frozendict({(mid,3): 1}): 8.0, frozendict({(mid,0): 1}): 2.0}})

    def test_expr3(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return 2*model.x[1] + 3*model.x[1] + 4*(model.x[1]+model.x[2])
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        mid=id(self.instance)
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        del rep[-1]
        self.assertEqual(rep,
                    {1: {frozendict({(mid,0): 1}):9.0, frozendict({(mid,1): 1}):4.0}})

    def test_expr4(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return 1.2 + 2*model.x[1] + 3*model.x[1]
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        mid=id(self.instance)
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        del rep[-1]
        self.assertEqual(rep, {0:frozendict({None:1.2}), 1: {frozendict({(mid,0): 1}): 5.0}})

    def test_expr5(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return model.x[1]*(model.x[1]+model.x[2]) + model.x[2]*(model.x[1]+3.0*model.x[3]*model.x[3])
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        mid=id(self.instance)
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        del rep[-1]
        self.assertEqual(rep,
                {2: {frozendict({(mid,0):2}):1.0, frozendict({(mid,0):1, (mid,1):1}):2.0}, 3:{frozendict({(mid,1):1, (mid,2):2}):3.0}})

    def test_expr6(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return 2.0*(1.2 + 2*model.x[1] + 3*model.x[1]) + 3.0*(1.0+model.x[1])
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        mid=id(self.instance)
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        del rep[-1]
        self.assertEqual(rep,
            {0:frozendict({None:5.4}), 1: {frozendict({(mid,0): 1}): 13.0}})

    def test_expr7(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return summation(model.p, model.x)/2.0
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        mid=id(self.instance)
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        del rep[-1]
        self.assertEqual(rep,
            {1: {frozendict({(mid,2): 1}): 3.0, frozendict({(mid,1): 1}): 2.0, frozendict({(mid,3): 1}): 4.0, frozendict({(mid,0): 1}): 1.0}})

    def test_expr8(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        self.model.y = Var(initialize=2.0)
        def obj_rule(model):
            return summation(model.p, model.x)/model.y
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        self.instance.y.fixed = True
        self.instance.y.reset()
        mid=id(self.instance)
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        del rep[-1]
        self.assertEqual(rep,
            {1: {frozendict({(mid,2): 1}): 3.0, frozendict({(mid,1): 1}): 2.0, frozendict({(mid,3): 1}): 4.0, frozendict({(mid,0): 1}): 1.0}})

    def test_expr9(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        self.model.y = Var(initialize=1.0)
        def obj_rule(model):
            return summation(model.p, model.x)/(1+model.y)
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        self.instance.y.fixed = True
        self.instance.y.reset()
        mid=id(self.instance)
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        del rep[-1]
        self.assertEqual(rep,
            {1: {frozendict({(mid,2): 1}): 3.0, frozendict({(mid,1): 1}): 2.0, frozendict({(mid,3): 1}): 4.0, frozendict({(mid,0): 1}): 1.0}})

    def test_expr_rational_summation(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        self.model.y = Var(initialize=1.0)
        def obj_rule(model):
            return summation(model.p, model.x)/(1+model.y)
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        self.assertTrue(len(rep) == 2 and None in rep and -1 in rep)
        self.assertEqual(len(rep[-1]), 5)
        #self.assertTrue(self.instance.y in rep[-1].values())

    def test_expr_rational(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        self.model.y = Var(initialize=1.0)
        def obj_rule(model):
            return (1.25+model.x[1]+1/model.y) + (2.0+model.x[2])/model.y

        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        #rep[None].pprint()
        self.assertTrue(len(rep) == 2 and None in rep and -1 in rep)
        self.assertEqual(len(rep[-1]), 3)
 
    def test_expr_rational_fixed(self):
        self.model.A = RangeSet(1,4)
        def p_init(i, model):
            return 2*i
        self.model.p = Param(self.model.A, initialize=p_init)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        self.model.y = Var(initialize=2.0)
        def obj_rule(model):
            return (1.25+model.x[1]+1/model.y) + (2.0+model.x[2])/model.y
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        self.instance.y.fixed = True
        mid=id(self.instance)
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(self.instance.obj[None].expr)
        self.assertEqual(len(rep), 3)
        self.assertTrue(-1 in rep and 0 in rep and 1 in rep)
        self.assertEqual(rep[0], {None: 2.75})
        self.assertEqual(rep[1], {frozendict({(mid,2): 1}): 0.5,
                                  frozendict({(mid,0): 1}): 1.0})

if __name__ == "__main__":
   unittest.main()




# a simple example illustrating piecewise linear
# constraints on the value of a variable X.
#
# the minimal objective function value is obtained
# with x=0. because there is a single breakpoint,
# the fixed point (where y=0) is assumed to be
# (0,0) - by adding the keyword "offset=y", this
# can be changed to (0,y). Thus, the optimal
# objective function value at x=0 is 0!

from coopr.pyomo import *

model = AbstractModel()

model.X = Var(bounds=(-5,5))

breakpoints = [0]
slopes =      [-1, 1]

model.Z = Piecewise(breakpoints=breakpoints,
                    slopes=slopes,
                    offset=0,
                    value=model.X)

model.obj = Objective(rule = lambda model: model.Z, sense=minimize)
